# -*- coding: utf-8 -*-

from django.conf import settings
from django.contrib.auth.models import AnonymousUser
from django.core import mail
from django.core.files.uploadedfile import UploadedFile
from django.test import tag, Client
from django.urls import reverse
from django.utils import timezone

import datetime
import pytz
import os
import uuid
from io import BytesIO

from accounts.models import User, Group
from accounts.utils import update_user
from guichet.models import (
    PaypalAccount,
    Event,
    Manager,
    UserManager,
    GroupManager,
    Registration,
    Ticket,
    Option,
    Info_Sup,
    Info_Sup_Choice,
    Info_Sup_Choice_Choices,
    Info_Sup_Int,
    Info_Sup_Float,
    Info_Sup_File,
    Info_Sup_Recv,
    Info_Sup_Recv_File,
    LydiaAccount,
)
from guichet.utils import maj_state_registration
from guichet.tests import CustomTestCase


class CleanUp(object):
    def tearDown(self):
        path_export = os.path.join(settings.MEDIA_ROOT, "protected", "export_archive")
        if os.path.isdir(path_export):
            for i in os.listdir(path_export):
                os.remove(os.path.join(path_export, i))
        for info_file in Info_Sup_Recv_File.objects.all():
            if info_file.value:
                os.remove(os.path.join(settings.MEDIA_ROOT, info_file.value.path))
        for ticket in Ticket.objects.all():
            if ticket.design:
                ticket.design = None
                # delete design file in media folder
                ticket.save()


class TestAccesGuichet(CleanUp, CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    # test de la page principale et de l'accès au évènement

    @tag("low", "local", "exte", "ldap")
    def test_acces_event_publique(self):
        event = Event.objects.get(pk=1)
        for old in (False, True):
            if old:
                event.start_date = timezone.now() - datetime.timedelta(days=2)
                event.end_date = timezone.now() - datetime.timedelta(days=1)
            else:
                event.start_date = timezone.now() + datetime.timedelta(days=1)
                event.end_date = timezone.now() + datetime.timedelta(days=2)
            event.save()

            self.client.logout()
            response = self.client.get(reverse("list-events"))
            if old:
                self.assertNotContains(response, event.name, status_code=200)
            else:
                self.assertContains(response, event.name, status_code=200)
            response2 = self.client.get(reverse("list-past-events"))
            self.assertRedirects(response2, reverse("login") + "?next=" + reverse("list-past-events"))
            response3 = self.client.get(reverse("show-event", args=[event.short_url]))
            self.assertEqual(response3.status_code, 200)
            for user in User.objects.filter(is_anonyme=False):
                self.client.force_login(user)
                response = self.client.get(reverse("list-events"))
                response2 = self.client.get(reverse("list-past-events"))
                if old:
                    self.assertNotContains(response, event.name, status_code=200)
                    self.assertContains(response2, event.name, status_code=200)
                else:
                    self.assertContains(response, event.name, status_code=200)
                    self.assertNotContains(response2, event.name, status_code=200)
                response3 = self.client.get(reverse("show-event", args=[event.short_url]))
                self.assertEqual(response3.status_code, 200)

    @tag("low", "local", "exte", "ldap")
    def test_acces_event_login(self):
        event = Event.objects.get(pk=1)
        event.event_confidentiality = Event.EVENT_LOGIN
        event.save()
        for old in (False, True):
            if old:
                event.start_date = timezone.now() - datetime.timedelta(days=2)
                event.end_date = timezone.now() - datetime.timedelta(days=1)
            else:
                event.start_date = timezone.now() + datetime.timedelta(days=1)
                event.end_date = timezone.now() + datetime.timedelta(days=2)
            event.save()

            self.client.logout()

            response = self.client.get(reverse("list-events"))
            self.assertNotContains(response, event.name, status_code=200)
            response2 = self.client.get(reverse("list-past-events"))
            self.assertRedirects(response2, reverse("login") + "?next=" + reverse("list-past-events"))
            response3 = self.client.get(reverse("show-event", args=[event.short_url]))
            self.assertRedirects(response3, reverse("login") + "?next=" + reverse("show-event", args=[event.short_url]))
            for user in User.objects.filter(is_anonyme=False):
                self.client.force_login(user)
                response = self.client.get(reverse("list-events"))
                response2 = self.client.get(reverse("list-past-events"))
                if old:
                    self.assertNotContains(response, event.name, status_code=200)
                    self.assertContains(response2, event.name, status_code=200)
                else:
                    self.assertContains(response, event.name, status_code=200)
                    self.assertNotContains(response2, event.name, status_code=200)
                response3 = self.client.get(reverse("show-event", args=[event.short_url]))
                self.assertEqual(response3.status_code, 200)

    @tag("low", "local", "exte", "ldap")
    def test_acces_event_restriction(self):
        eventnew = Event(
            name="TestRestrictEventNew",
            start_date=timezone.now() + datetime.timedelta(days=1),
            end_date=timezone.now() + datetime.timedelta(days=2),
            paypal_account=PaypalAccount.default_paypal_account(),
            lydia_account=LydiaAccount.default_lydia_account(),
            event_confidentiality=Event.EVENT_PRIVE,
            email_event="test@billetterie.inpt.fr",
        )
        eventnew.save()
        eventold = Event(
            name="TestRestrictEventOld",
            start_date=timezone.now() - datetime.timedelta(days=2),
            end_date=timezone.now() - datetime.timedelta(days=1),
            paypal_account=PaypalAccount.default_paypal_account(),
            lydia_account=LydiaAccount.default_lydia_account(),
            event_confidentiality=Event.EVENT_PRIVE,
            email_event="test@billetterie.inpt.fr",
        )
        eventold.save()
        for event, old in [(eventnew, False), (eventold, True)]:
            ticket = Ticket(
                event=event,
                name="Testrestrict",
                price=10,
                id=1,
                online_payment=False,
                allowed_external=False,
                required_school=Ticket.SCHOOL_INP,
                required_ae=True,
                admin_ticket=False,
            )
            ticket.save()
            group = Group.objects.get(cn="test-inp")
            ticket.groups.add(group)
            ticket.save()
            self.client.logout()

            response = self.client.get(reverse("list-events"))
            self.assertNotContains(response, event.name, status_code=200)
            response2 = self.client.get(reverse("list-past-events"))
            self.assertRedirects(response2, reverse("login") + "?next=" + reverse("list-past-events"))
            response3 = self.client.get(reverse("show-event", args=[event.short_url]))
            self.assertRedirects(response3, reverse("login") + "?next=" + reverse("show-event", args=[event.short_url]))
            for user in User.objects.filter(is_anonyme=False):
                self.client.force_login(user)
                response = self.client.get(reverse("list-events"))
                response2 = self.client.get(reverse("list-past-events"))
                response3 = self.client.get(reverse("show-event", args=[event.short_url]))
                if (user.is_superuser) or (user in group.users.all() and user.is_adherent_ae):
                    if old:
                        self.assertNotContains(response, event.name, status_code=200)
                        self.assertContains(response2, event.name, status_code=200)
                    else:
                        self.assertContains(response, event.name, status_code=200)
                        self.assertNotContains(response2, event.name, status_code=200)
                    self.assertEqual(response3.status_code, 200)
                else:
                    self.assertNotContains(response, event.name, status_code=200)
                    self.assertNotContains(response2, event.name, status_code=200)
                    self.assertEqual(response3.status_code, 404)

    @tag("low", "local", "exte", "ldap")
    def test_acces_event_privee(self):
        event = Event.objects.get(pk=1)
        event.event_confidentiality = Event.EVENT_HIDDEN
        event.save()
        for old in (False, True):
            if old:
                event.start_date = timezone.now() - datetime.timedelta(days=2)
                event.end_date = timezone.now() - datetime.timedelta(days=1)
            else:
                event.start_date = timezone.now() + datetime.timedelta(days=1)
                event.end_date = timezone.now() + datetime.timedelta(days=2)
            event.save()

            self.client.logout()
            response = self.client.get(reverse("list-events"))
            self.assertNotContains(response, event.name, status_code=200)
            response2 = self.client.get(reverse("list-past-events"))
            self.assertRedirects(response2, reverse("login") + "?next=" + reverse("list-past-events"))
            response3 = self.client.get(reverse("show-event", args=[event.short_url]))
            self.assertEqual(response3.status_code, 200)
            for user in User.objects.filter(is_anonyme=False):
                self.client.force_login(user)
                response = self.client.get(reverse("list-events"))
                response2 = self.client.get(reverse("list-past-events"))
                if user.is_superuser or event.is_manager(user):
                    if old:
                        self.assertNotContains(response, event.name, status_code=200)
                        self.assertContains(response2, event.name, status_code=200)
                    else:
                        self.assertContains(response, event.name, status_code=200)
                        self.assertNotContains(response2, event.name, status_code=200)
                else:
                    self.assertNotContains(response, event.name, status_code=200)
                    self.assertNotContains(response2, event.name, status_code=200)
                response3 = self.client.get(reverse("show-event", args=[event.short_url]))
                self.assertEqual(response3.status_code, 200)


class TestManagerAccess(CleanUp, CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    @tag("fast", "local", "exte", "ldap")
    def test_decorator(self):
        # test différence entre non manager et manager
        event = Event.objects.get(pk=1)
        response = self.client.get(reverse("show-stats", args=[event.short_url]))
        self.assertRedirects(response, reverse("login") + "?next=" + reverse("show-stats", args=[event.short_url]))
        for user in User.objects.filter(is_anonyme=False):
            self.client.force_login(user)
            response = self.client.get(reverse("show-stats", args=[event.short_url]))
            if user.is_superuser or event.is_manager(user):
                self.assertEqual(response.status_code, 200)
            else:
                self.assertEqual(response.status_code, 403)

    @tag("fast", "local", "ldap")
    def test_decorator_level_access(self):
        # test les niveaux de manager
        event = Event.objects.get(pk=1)
        access_right = UserManager.objects.filter(event=event).first()
        user = access_right.user
        for acl in [Manager.FULL_VIEW, Manager.ENTRY_MANAGER, Manager.EVENT_MANAGER, Manager.FULL_MANAGER]:
            access_right.access = acl
            access_right.save()
            self.client.force_login(user)
            # stats == tous les managers
            response = self.client.get(reverse("show-stats", args=[event.short_url]))
            self.assertEqual(response.status_code, 200)
            # entry == ENTRY_MANAGER
            response2 = self.client.post(reverse("check-registration", args=[event.short_url]))
            if acl >= Manager.ENTRY_MANAGER:
                self.assertEqual(response2.status_code, 200)
            else:
                self.assertRedirects(response2, reverse("show-event", args=[event.short_url]))
            # add ticket == EVENT_MANAGER
            response3 = self.client.post(reverse("add-ticket", args=[event.short_url]))
            if acl >= Manager.EVENT_MANAGER:
                self.assertEqual(response3.status_code, 200)
            else:
                self.assertRedirects(response3, reverse("show-event", args=[event.short_url]))
            # add manager == FULL_MANAGER
            response4 = self.client.post(reverse("list-managers", args=[event.short_url]))
            if acl >= Manager.FULL_MANAGER:
                self.assertEqual(response4.status_code, 200)
            else:
                self.assertRedirects(response4, reverse("show-event", args=[event.short_url]))

    @tag("fast", "local", "nologin")
    def test_manager_level(self):
        # test multi acl
        event = Event.objects.get(pk=1)
        user1 = User.objects.get(login_inp_net="testensat")
        user2 = User.objects.get(login_inp_net="testasept")
        group = Group.objects.get(pk=1)
        self.assertTrue(group in user1.groupes.all())
        self.assertTrue(group in user2.groupes.all())
        self.assertTrue(group in user1.managed_clubs.all())
        self.assertFalse(group in user2.managed_clubs.all())
        list_acl = [
            Manager.NO_ACCESS,
            Manager.FULL_VIEW,
            Manager.ENTRY_MANAGER,
            Manager.EVENT_MANAGER,
            Manager.FULL_MANAGER,
        ]
        for user in [user1, user2]:
            for user_acl in list_acl:
                UserManager.objects.filter(event=event, user=user).delete()
                if user_acl != Manager.NO_ACCESS:
                    UserManager(event=event, user=user, access=user_acl).save()
                for group_acl in list_acl:
                    GroupManager.objects.filter(event=event, group=group, bureau_only=False).delete()
                    if group_acl != Manager.NO_ACCESS:
                        GroupManager(event=event, group=group, bureau_only=False, access=group_acl).save()
                    for bureau_acl in list_acl:
                        GroupManager.objects.filter(event=event, group=group, bureau_only=True).delete()
                        if bureau_acl != Manager.NO_ACCESS:
                            GroupManager(event=event, group=group, bureau_only=True, access=bureau_acl).save()
                        if group in user.managed_clubs.all():
                            self.assertEqual(event.get_access_manager(user), max(user_acl, group_acl, bureau_acl))
                        else:
                            self.assertEqual(event.get_access_manager(user), max(user_acl, group_acl))

        self.assertEqual(event.get_access_manager(User.objects.filter(is_superuser=True).first()), Manager.FULL_MANAGER)
        self.assertEqual(event.get_access_manager(AnonymousUser()), Manager.NO_ACCESS)


class TestAdminAccess(CleanUp, CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    @tag("low", "ldap")
    def test_access(self):
        user = User.objects.get(pk=7, is_superuser=True)
        self.client.force_login(user)
        response = self.client.get(reverse("admin:index"))
        self.assertEqual(response.status_code, 200)


class TestCreateEvent(CleanUp, CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    def setUp(self):
        self.tz = pytz.timezone("Europe/Paris")
        self.eventform = {
            "name": "NewEvent",
            "start_date": (timezone.now() + datetime.timedelta(days=1)).astimezone(self.tz).strftime("%Y-%m-%d %H:%M"),
            "end_date": (timezone.now() + datetime.timedelta(days=2)).astimezone(self.tz).strftime("%Y-%m-%d %H:%M"),
            "description": "ma description",
            "email_event": "new_event@billetterie.inpt.fr",
            "email_message": "You have an email !",
            "other_payment_description": "On accepte en nature.",
            "terms_of_use": "Vous devez savoir coder sauf en java et php.",
            "limit_godson": 0,
            "limit_ticket": 0,
            "paypal_account": PaypalAccount.default_paypal_account().pk,
            "lydia_account": 1,
            "event_confidentiality": Event.EVENT_PUBLIC,
            "conditions_field": True,
        }

    @tag("low", "ldap")
    def test_accessForm(self):
        response = self.client.get(reverse("paypal-account"))
        response2 = self.client.get(reverse("add-event"))
        self.assertRedirects(response, reverse("login") + "?next=" + reverse("paypal-account"))
        self.assertRedirects(response2, reverse("login") + "?next=" + reverse("add-event"))
        for user in User.objects.filter(is_anonyme=False):
            update_user(user)
            self.client.force_login(user)
            response = self.client.get(reverse("paypal-account"))
            response2 = self.client.get(reverse("add-event"))
            if user.is_superuser or len(user.managed_clubs.all()) > 0:
                self.assertEqual(response.status_code, 200)
                self.assertEqual(response2.status_code, 200)
            else:
                self.assertEqual(response.status_code, 403)
                self.assertEqual(response2.status_code, 403)

    @tag("fast", "local")
    def test_createPaypalAccount(self):
        user = User.objects.get(email="admin@billetterie.inpt.fr")
        self.client.force_login(user)
        form = {
            "email": "new_paypal_account@billetterie.inpt.fr",
            "description": "ma description",
        }
        response = self.client.post(reverse("paypal-account"), form)
        self.assertRedirects(response, reverse("add-event"))
        accounts = PaypalAccount.objects.filter(
            email="new_paypal_account@billetterie.inpt.fr", description="ma description", no_account=False
        )
        self.assertEqual(len(accounts), 1)

    @tag("fast", "local")
    def test_createEvent(self):
        user = User.objects.get(email="admin@billetterie.inpt.fr")
        self.client.force_login(user)
        response = self.client.post(reverse("add-event"), self.eventform)
        eventinfo = self.eventform.copy()
        del eventinfo["start_date"]
        del eventinfo["end_date"]
        del eventinfo["conditions_field"]
        event = Event.objects.filter(**eventinfo)
        self.assertEqual(len(event), 1)
        event = event.first()
        self.assertRedirects(response, reverse("show-event", args=[event.short_url]))
        self.assertEqual(event.start_date.astimezone(self.tz).strftime("%Y-%m-%d %H:%M"), self.eventform["start_date"])
        self.assertEqual(event.end_date.astimezone(self.tz).strftime("%Y-%m-%d %H:%M"), self.eventform["end_date"])
        user_acl = UserManager.objects.filter(event=event, user=user, access=Manager.FULL_MANAGER)
        self.assertEqual(len(user_acl), 1)

    @tag("fast", "local")
    def test_createEventWithBugName(self):
        user = User.objects.get(email="admin@billetterie.inpt.fr")
        self.client.force_login(user)
        eventinfo = self.eventform.copy()
        eventinfo["name"] = "add"
        response = self.client.post(reverse("add-event"), eventinfo)
        del eventinfo["start_date"]
        del eventinfo["end_date"]
        del eventinfo["conditions_field"]
        event = Event.objects.filter(**eventinfo)
        self.assertEqual(len(event), 1)
        event = event.first()
        self.assertRedirects(response, reverse("show-event", args=[event.short_url]))
        self.assertEqual(event.start_date.astimezone(self.tz).strftime("%Y-%m-%d %H:%M"), self.eventform["start_date"])
        self.assertEqual(event.end_date.astimezone(self.tz).strftime("%Y-%m-%d %H:%M"), self.eventform["end_date"])
        user_acl = UserManager.objects.filter(event=event, user=user, access=Manager.FULL_MANAGER)
        self.assertEqual(len(user_acl), 1)

    @tag("fast", "local")
    def test_wrongdateEvent(self):
        user = User.objects.get(email="admin@billetterie.inpt.fr")
        self.client.force_login(user)
        eventform = self.eventform.copy()
        eventform["start_date"] = (
            (timezone.now() + datetime.timedelta(days=3)).astimezone(self.tz).strftime("%Y-%m-%d %H:%M")
        )
        response = self.client.post(reverse("add-event"), eventform)
        event = Event.objects.filter(name=eventform["name"])
        self.assertEqual(len(event), 0)
        self.assertEqual(response.status_code, 200)
        self.assertFormError(response, "form", "end_date", "La date de fin doit être après le début de l'évènement.")

    @tag("fast", "local")
    def test_NoConditionEvent(self):
        user = User.objects.get(email="admin@billetterie.inpt.fr")
        self.client.force_login(user)
        eventform = self.eventform.copy()
        del eventform["conditions_field"]
        response = self.client.post(reverse("add-event"), eventform)
        event = Event.objects.filter(name=eventform["name"])
        self.assertEqual(len(event), 0)
        self.assertEqual(response.status_code, 200)

    @tag("fast", "local")
    def test_wrongformEvent(self):
        user = User.objects.get(email="admin@billetterie.inpt.fr")
        self.client.force_login(user)
        eventform = self.eventform.copy()
        eventform["event_confidentiality"] = ""
        response = self.client.post(reverse("add-event"), eventform)
        event = Event.objects.filter(name=eventform["name"])
        self.assertEqual(len(event), 0)
        self.assertEqual(response.status_code, 200)
        self.assertFormError(response, "form", "event_confidentiality", "Ce champ est obligatoire.")

    @tag("fast", "local")
    def test_caneditEvent(self):
        user = User.objects.get(email="admin@billetterie.inpt.fr")
        self.client.force_login(user)
        self.client.post(reverse("add-event"), self.eventform)
        eventinfo = self.eventform.copy()
        del eventinfo["start_date"]
        del eventinfo["end_date"]
        del eventinfo["conditions_field"]
        event = Event.objects.filter(**eventinfo)
        self.assertEqual(len(event), 1)
        event = event.first()
        user2 = User.objects.get(login_inp_net="testensat")
        user3 = User.objects.get(login_inp_net="testone")
        UserManager(event=event, user=user2, access=Manager.FULL_MANAGER).save()

        self.client.force_login(user2)
        response = self.client.get(reverse("edit-event", args=[event.short_url]))
        self.assertEqual(response.status_code, 200)
        self.client.force_login(user3)
        response2 = self.client.get(reverse("edit-event", args=[event.short_url]))
        self.assertEqual(response2.status_code, 403)

    @tag("fast", "local")
    def test_editEventSetNoOnlinePayement(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        eventinfo = self.eventform.copy()
        eventinfo["paypal_account"] = PaypalAccount.objects.filter(no_account=False).first()
        eventinfo["lydia_account"] = LydiaAccount.objects.filter(no_account=False).first()
        eventinfo["start_date"] = (timezone.now() + datetime.timedelta(days=1)).astimezone(self.tz)
        eventinfo["end_date"] = (timezone.now() + datetime.timedelta(days=2)).astimezone(self.tz)
        del eventinfo["conditions_field"]
        event = Event(**eventinfo)
        event.save()
        UserManager(event=event, user=user, access=Manager.FULL_MANAGER).save()
        ticket = Ticket(
            event=event, name="TicketTestOnline", online_payment=True, allowed_external=True, required_ae=False, id=1
        )
        ticket.save()

        response = self.client.post(reverse("edit-event", args=[event.short_url]), self.eventform)
        self.assertRedirects(response, reverse("show-event", args=[event.short_url]))
        new_ticket = Ticket.objects.get(pk=ticket.pk)
        self.assertFalse(new_ticket.online_payment)
        self.assertTrue(Event.objects.get(pk=event.pk).paypal_account.no_account)


class TestDelecteEvent(CleanUp, CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    @tag("fast", "local")
    def test_deleteEvent(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)

        response = self.client.post(reverse("delete-event", args=[event.short_url]))
        self.assertRedirects(response, reverse("list-events"))
        self.assertFalse(Event.objects.filter(pk=1).exists())

    @tag("fast", "local")
    def test_deleteEventwithPreReservation(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        Registration(owner=user, ticket=Ticket.objects.filter(event=event).first(), conditions_accepted=True).save()

        response = self.client.post(reverse("delete-event", args=[event.short_url]))
        self.assertRedirects(response, reverse("list-events"))
        self.assertFalse(Event.objects.filter(pk=1).exists())

    @tag("fast", "local")
    def test_deleteEventwithReservation(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        Registration(
            owner=user, ticket=Ticket.objects.filter(event=event).first(), conditions_accepted=True, paid=True
        ).save()

        response = self.client.post(reverse("delete-event", args=[event.short_url]))
        self.assertRedirects(response, reverse("show-event", args=[event.short_url]))
        self.assertTrue(Event.objects.filter(pk=1).exists())


class TestCreateTicket(CleanUp, CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    def setUp(self):
        self.tz = pytz.timezone("Europe/Paris")
        self.openticket = {
            "name": "OpenTicket",
            "short_notes": "short description",
            "long_notes": "long description",
            "price": 10,
            "online_payment": False,
            "limit": 0,
            "admin_ticket": False,
            "allow_anonyme": False,
        }
        self.restrictedticket = {
            "name": "RestrictedTicket",
            "short_notes": "short description",
            "long_notes": "long description",
            "price": 10,
            "online_payment": False,
            "limit": 0,
            "year_choice": [Ticket.YEAR_1A, Ticket.YEAR_2A],
            "required_school": Ticket.SCHOOL_INP,
            "required_ae": True,
            "allowed_godson": False,
            "admin_ticket": False,
            "allow_anonyme": False,
        }

    @tag("fast", "local")
    def test_choicetypeticket(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        response = self.client.get(reverse("add-ticket", args=[event.short_url]))
        self.assertEqual(response.status_code, 200)

    @tag("fast", "local")
    def test_createOpenTicket(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        # get
        response = self.client.get(reverse("add-open-ticket", args=[event.short_url]))
        self.assertEqual(response.status_code, 200)
        # post
        response2 = self.client.post(reverse("add-open-ticket", args=[event.short_url]), self.openticket)
        tickets = Ticket.objects.filter(event=event, **self.openticket)
        self.assertEqual(len(tickets), 1)
        ticket = tickets.first()
        self.assertRedirects(response2, reverse("details-ticket", args=[event.short_url, ticket.id]))
        self.assertTrue(ticket.allowed_external)
        # billet in list
        response3 = self.client.get(reverse("list-tickets", args=[event.short_url]))
        self.assertEqual(response3.status_code, 200)
        self.assertContains(response3, self.openticket["name"])

    @tag("fast", "local")
    def test_createOpenTicketWithOnlyPaymentWithoutPaypalAccount(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        self.assertTrue(event.paypal_account.no_account)
        openticketform = self.openticket.copy()
        openticketform["online_payment"] = True
        # post
        response2 = self.client.post(reverse("add-open-ticket", args=[event.short_url]), openticketform)
        self.assertFalse(Ticket.objects.filter(event=event, **openticketform).exists())
        self.assertEqual(len(Ticket.objects.filter(event=event, **self.openticket)), 1)
        ticket = Ticket.objects.get(event=event, **self.openticket)
        self.assertRedirects(response2, reverse("details-ticket", args=[event.short_url, ticket.id]))
        self.assertTrue(ticket.allowed_external)

    @tag("fast", "local")
    def test_createOpenTicketWithOnlyPaymentWithPaypalAccount(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        event.paypal_account = PaypalAccount.objects.filter(no_account=False).first()
        event.save()
        self.client.force_login(user)
        self.assertFalse(event.paypal_account.no_account)
        openticketform = self.openticket.copy()
        openticketform["online_payment"] = True
        # post
        response2 = self.client.post(reverse("add-open-ticket", args=[event.short_url]), openticketform)
        self.assertFalse(Ticket.objects.filter(event=event, **self.openticket).exists())
        self.assertEqual(len(Ticket.objects.filter(event=event, **openticketform)), 1)
        ticket = Ticket.objects.get(event=event, **openticketform)
        self.assertRedirects(response2, reverse("details-ticket", args=[event.short_url, ticket.id]))
        self.assertTrue(ticket.allowed_external)

    @tag("fast", "local")
    def test_editOpenTicket(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        response = self.client.post(reverse("add-open-ticket", args=[event.short_url]), self.openticket)
        self.assertEqual(response.status_code, 302)
        tickets = Ticket.objects.filter(event=event, name=self.openticket["name"])
        self.assertEqual(len(tickets), 1)
        ticket = tickets.first()
        response2 = self.client.get(reverse("edit-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response2.status_code, 200)
        openticketform = self.openticket.copy()
        openticketform["online_payment"] = True
        response3 = self.client.post(reverse("edit-ticket", args=[event.short_url, ticket.id]), openticketform)
        self.assertRedirects(response3, reverse("details-ticket", args=[event.short_url, ticket.id]))
        self.assertFalse(Ticket.objects.get(pk=ticket.pk).online_payment)

    @tag("fast", "local")
    def test_editOpenTicketIsValid(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        response = self.client.post(reverse("add-open-ticket", args=[event.short_url]), self.restrictedticket)
        self.assertEqual(response.status_code, 302)
        tickets = Ticket.objects.filter(event=event, name=self.restrictedticket["name"])
        self.assertEqual(len(tickets), 1)
        ticket = tickets.first()
        # push invalid form
        response2 = self.client.post(reverse("edit-ticket", args=[event.short_url, ticket.id]), {})
        self.assertEqual(response2.status_code, 200)  # not redirect
        # push invalid min/max_purshase_date
        openform = self.openticket.copy()
        openform["max_purchase_date"] = (
            (event.end_date - datetime.timedelta(days=2)).astimezone(self.tz).strftime("%Y-%m-%d %H:%M")
        )
        openform["min_purchase_date"] = (
            (event.end_date - datetime.timedelta(days=1)).astimezone(self.tz).strftime("%Y-%m-%d %H:%M")
        )
        response3 = self.client.post(reverse("edit-ticket", args=[event.short_url, ticket.id]), openform)
        self.assertEqual(response3.status_code, 200)
        self.assertFormError(
            response3, "form", "min_purchase_date", "La date limite de vente doit être après la date de début de vente"
        )

    @tag("fast", "local")
    def test_createRestrictedTicket(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        # get
        response = self.client.get(reverse("add-restricted-ticket", args=[event.short_url]))
        self.assertEqual(response.status_code, 200)
        # post
        response2 = self.client.post(reverse("add-restricted-ticket", args=[event.short_url]), self.restrictedticket)
        restrictedticket = self.restrictedticket.copy()
        del restrictedticket["year_choice"]
        tickets = Ticket.objects.filter(event=event, **restrictedticket)
        self.assertEqual(len(tickets), 1)
        ticket = tickets.first()
        self.assertRedirects(response2, reverse("details-ticket", args=[event.short_url, ticket.id]))
        self.assertFalse(ticket.allowed_external)
        for promo, text in Ticket.YEAR_CHOICES:
            if promo in self.restrictedticket["year_choice"]:
                self.assertIn(promo, ticket.get_year_choice())
            else:
                self.assertNotIn(promo, ticket.get_year_choice())
        # billet in list
        response3 = self.client.get(reverse("list-tickets", args=[event.short_url]))
        self.assertEqual(response3.status_code, 200)
        self.assertContains(response3, restrictedticket["name"])

    @tag("fast", "local")
    def test_editRestrictedTicket(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        response = self.client.post(reverse("add-restricted-ticket", args=[event.short_url]), self.restrictedticket)
        self.assertEqual(response.status_code, 302)
        tickets = Ticket.objects.filter(event=event, name=self.restrictedticket["name"])
        self.assertEqual(len(tickets), 1)
        ticket = tickets.first()
        response2 = self.client.get(reverse("edit-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response2.status_code, 200)
        restrictedform = self.restrictedticket.copy()
        # set max_purchase_date later that event
        restrictedform["max_purchase_date"] = (
            (event.end_date + datetime.timedelta(days=1)).astimezone(self.tz).strftime("%Y-%m-%d %H:%M")
        )
        response3 = self.client.post(reverse("edit-ticket", args=[event.short_url, ticket.id]), restrictedform)
        self.assertRedirects(response3, reverse("details-ticket", args=[event.short_url, ticket.id]))
        self.assertIs(Ticket.objects.get(pk=ticket.pk).max_purchase_date, None)

    @tag("fast", "local")
    def test_editRestrictedTicketIsValid(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        response = self.client.post(reverse("add-restricted-ticket", args=[event.short_url]), self.restrictedticket)
        self.assertEqual(response.status_code, 302)
        tickets = Ticket.objects.filter(event=event, name=self.restrictedticket["name"])
        self.assertEqual(len(tickets), 1)
        ticket = tickets.first()
        # push invalid form
        response2 = self.client.post(reverse("edit-ticket", args=[event.short_url, ticket.id]), {})
        self.assertEqual(response2.status_code, 200)  # not redirect
        # push invalid min/max_purshase_date
        restrictedform = self.restrictedticket.copy()
        restrictedform["max_purchase_date"] = (
            (event.end_date - datetime.timedelta(days=2)).astimezone(self.tz).strftime("%Y-%m-%d %H:%M")
        )
        restrictedform["min_purchase_date"] = (
            (event.end_date - datetime.timedelta(days=1)).astimezone(self.tz).strftime("%Y-%m-%d %H:%M")
        )
        response3 = self.client.post(reverse("edit-ticket", args=[event.short_url, ticket.id]), restrictedform)
        self.assertEqual(response3.status_code, 200)
        self.assertFormError(
            response3, "form", "min_purchase_date", "La date limite de vente doit être après la date de début de vente"
        )

    @tag("fast", "local")
    def test_deleteTicket(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        response = self.client.post(reverse("add-restricted-ticket", args=[event.short_url]), self.restrictedticket)
        self.assertEqual(response.status_code, 302)
        tickets = Ticket.objects.filter(event=event, name=self.restrictedticket["name"])
        self.assertEqual(len(tickets), 1)
        ticket = tickets.first()
        response2 = self.client.post(reverse("delete-ticket", args=[event.short_url, ticket.id]))
        self.assertRedirects(response2, reverse("list-tickets", args=[event.short_url]))
        self.assertFalse(Ticket.objects.filter(event=event, id=ticket.id).exists())

    @tag("fast", "local")
    def test_deleteTicketWithPreRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        response = self.client.post(reverse("add-restricted-ticket", args=[event.short_url]), self.restrictedticket)
        self.assertEqual(response.status_code, 302)
        tickets = Ticket.objects.filter(event=event, name=self.restrictedticket["name"])
        self.assertEqual(len(tickets), 1)
        ticket = tickets.first()
        Registration(owner=user, ticket=ticket).save()
        response2 = self.client.post(reverse("delete-ticket", args=[event.short_url, ticket.id]))
        self.assertRedirects(response2, reverse("list-tickets", args=[event.short_url]))
        self.assertFalse(Ticket.objects.filter(event=event, id=ticket.id).exists())

    @tag("fast", "local", "test")
    def test_deleteTicketWithRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        response = self.client.post(reverse("add-restricted-ticket", args=[event.short_url]), self.restrictedticket)
        self.assertEqual(response.status_code, 302)
        tickets = Ticket.objects.filter(event=event, name=self.restrictedticket["name"])
        self.assertEqual(len(tickets), 1)
        ticket = tickets.first()
        Registration(owner=user, ticket=ticket, paid=True).save()
        response2 = self.client.post(reverse("delete-ticket", args=[event.short_url, ticket.id]))
        self.assertRedirects(response2, reverse("list-tickets", args=[event.short_url]))
        self.assertTrue(Ticket.objects.filter(event=event, id=ticket.id).exists())


class TestCreateInfoSup(CleanUp, CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    def setUp(self):
        self.tz = pytz.timezone("Europe/Paris")
        self.infoSup = {
            "name": "infosupTest",
            "description": "info description",
            "after_payement": False,
            "type_info": Info_Sup.INFO_SIMPLE,
            "info_validate": False,
            "info_editable": False,
        }
        self.infoSupChoice = {
            "choices-TOTAL_FORMS": 2,
            "choices-INITIAL_FORMS": 0,
            "choices-MIN_NUM_FORMS": 1,
            "choices-MAX_NUM_FORMS": 1000,
            "choices-0-name": "TEST1",
            "choices-1-name": "TEST2",
        }

    @tag("fast", "local")
    def test_createInfoSupSimple(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # get choice
        response = self.client.get(reverse("infosup-choices-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response.status_code, 200)
        # get form
        response2 = self.client.get(
            reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_SIMPLE])
        )
        self.assertEqual(response2.status_code, 200)
        # post form
        response3 = self.client.post(
            reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_SIMPLE]), self.infoSup
        )
        self.assertRedirects(response3, reverse("details-ticket", args=[event.short_url, ticket.id]))
        infosups = Info_Sup.objects.filter(ticket=ticket, **self.infoSup)
        self.assertEqual(len(infosups), 1)
        # verify infosup in template
        response4 = self.client.get(reverse("details-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response4.status_code, 200)
        self.assertContains(response4, infosups.first().name)

    @tag("fast", "local")
    def test_createInfoSupShortText(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # get form
        self.infoSup["type_info"] = Info_Sup.INFO_SHORT_TEXT
        response = self.client.get(
            reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_SHORT_TEXT])
        )
        self.assertEqual(response.status_code, 200)
        # post form
        response3 = self.client.post(
            reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_SHORT_TEXT]), self.infoSup
        )
        self.assertRedirects(response3, reverse("details-ticket", args=[event.short_url, ticket.id]))
        infosups = Info_Sup.objects.filter(ticket=ticket, **self.infoSup)
        self.assertEqual(len(infosups), 1)
        # verify infosup in template
        response4 = self.client.get(reverse("details-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response4.status_code, 200)
        self.assertContains(response4, infosups.first().name)

    @tag("fast", "local")
    def test_createInfoSupLongText(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # get form
        self.infoSup["type_info"] = Info_Sup.INFO_LONG_TEXT
        response = self.client.get(
            reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_LONG_TEXT])
        )
        self.assertEqual(response.status_code, 200)
        # post form
        response3 = self.client.post(
            reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_LONG_TEXT]), self.infoSup
        )
        self.assertRedirects(response3, reverse("details-ticket", args=[event.short_url, ticket.id]))
        infosups = Info_Sup.objects.filter(ticket=ticket, **self.infoSup)
        self.assertEqual(len(infosups), 1)
        # verify infosup in template
        response4 = self.client.get(reverse("details-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response4.status_code, 200)
        self.assertContains(response4, infosups.first().name)

    @tag("fast", "local")
    def test_createInfoSupChoice(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # get form
        self.infoSup["type_info"] = Info_Sup.INFO_CHOICE
        response = self.client.get(
            reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_CHOICE])
        )
        self.assertEqual(response.status_code, 200)
        # post form
        d = self.infoSup.copy()
        d.update(self.infoSupChoice)
        response3 = self.client.post(
            reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_CHOICE]), d
        )
        self.assertRedirects(response3, reverse("details-ticket", args=[event.short_url, ticket.id]))
        infosups = Info_Sup.objects.filter(ticket=ticket, **self.infoSup)
        self.assertEqual(len(infosups), 1)
        info_choice = infosups.first().get_sup_type()
        self.assertEqual(len(info_choice.choices.all()), 2)
        self.assertTrue(info_choice.choices.filter(name="TEST1").exists())
        self.assertTrue(info_choice.choices.filter(name="TEST2").exists())
        # verify infosup in template
        response4 = self.client.get(reverse("details-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response4.status_code, 200)
        self.assertContains(response4, infosups.first().name)

    @tag("fast", "local")
    def test_createInfoSupInt(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # get form
        self.infoSup["type_info"] = Info_Sup.INFO_INT
        response = self.client.get(reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_INT]))
        self.assertEqual(response.status_code, 200)
        # post form
        d = self.infoSup.copy()
        d["max_value"] = 5
        response3 = self.client.post(
            reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_INT]), d
        )
        self.assertRedirects(response3, reverse("details-ticket", args=[event.short_url, ticket.id]))
        infosups = Info_Sup.objects.filter(ticket=ticket, **self.infoSup)
        self.assertEqual(len(infosups), 1)
        info_int = infosups.first().get_sup_type()
        self.assertEqual(info_int.max_value, 5)
        self.assertEqual(info_int.min_value, None)
        # verify infosup in template
        response4 = self.client.get(reverse("details-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response4.status_code, 200)
        self.assertContains(response4, infosups.first().name)

    @tag("fast", "local")
    def test_createInfoSupFloat(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # get form
        self.infoSup["type_info"] = Info_Sup.INFO_FLOAT
        response = self.client.get(
            reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_FLOAT])
        )
        self.assertEqual(response.status_code, 200)
        # post form
        d = self.infoSup.copy()
        d["min_value"] = 2.5
        response3 = self.client.post(
            reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_FLOAT]), d
        )
        self.assertRedirects(response3, reverse("details-ticket", args=[event.short_url, ticket.id]))
        infosups = Info_Sup.objects.filter(ticket=ticket, **self.infoSup)
        self.assertEqual(len(infosups), 1)
        info_float = infosups.first().get_sup_type()
        self.assertEqual(info_float.max_value, None)
        self.assertEqual(info_float.min_value, 2.5)
        # verify infosup in template
        response4 = self.client.get(reverse("details-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response4.status_code, 200)
        self.assertContains(response4, infosups.first().name)

    @tag("fast", "local")
    def test_createInfoSupFile(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # get form
        self.infoSup["type_info"] = Info_Sup.INFO_FILE
        response = self.client.get(reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_FILE]))
        self.assertEqual(response.status_code, 200)
        # post form
        d = self.infoSup.copy()
        d["file_name"] = "test"
        response3 = self.client.post(
            reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_FILE]), d
        )
        self.assertRedirects(response3, reverse("details-ticket", args=[event.short_url, ticket.id]))
        infosups = Info_Sup.objects.filter(ticket=ticket, **self.infoSup)
        self.assertEqual(len(infosups), 1)
        info_file = infosups.first().get_sup_type()
        self.assertEqual(info_file.file_name, "test")
        # verify infosup in template
        response4 = self.client.get(reverse("details-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response4.status_code, 200)
        self.assertContains(response4, infosups.first().name)

    @tag("fast", "local")
    def test_createInfoSupFloatWithBadLimit(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # post form
        self.infoSup["type_info"] = Info_Sup.INFO_FLOAT
        d = self.infoSup.copy()
        d["min_value"] = 2.5
        d["max_value"] = 1.5
        response = self.client.post(
            reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_FLOAT]), d
        )
        self.assertEqual(response.status_code, 200)
        infosups = Info_Sup.objects.filter(ticket=ticket, **self.infoSup)
        self.assertEqual(len(infosups), 0)

    @tag("fast", "local")
    def test_createInfoSupFloatWithBadOption(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # post form
        self.infoSup["type_info"] = Info_Sup.INFO_FLOAT
        d = self.infoSup.copy()
        d["min_value"] = 2.5
        d["info_editable"] = True
        response = self.client.post(
            reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_FLOAT]), d
        )
        self.assertEqual(response.status_code, 200)
        infosups = Info_Sup.objects.filter(ticket=ticket, **self.infoSup)
        self.assertEqual(len(infosups), 0)

    @tag("fast", "local")
    def test_createInfoSupWithRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        Registration(owner=user, ticket=ticket).save()
        # get form
        response = self.client.get(
            reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_SIMPLE])
        )
        self.assertRedirects(response, reverse("details-ticket", args=[event.short_url, ticket.id]))

    @tag("fast", "local")
    def test_editInfoSup(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        infosup = Info_Sup(ticket=ticket, **self.infoSup)
        infosup.save()
        self.client.force_login(user)
        # get form
        response = self.client.get(reverse("infosup-edit-ticket", args=[event.short_url, ticket.id, infosup.pk]))
        self.assertEqual(response.status_code, 200)
        # post
        infoform = self.infoSup.copy()
        infoform["after_payement"] = True
        response2 = self.client.post(
            reverse("infosup-edit-ticket", args=[event.short_url, ticket.id, infosup.pk]), infoform
        )
        self.assertRedirects(response2, reverse("details-ticket", args=[event.short_url, ticket.id]))
        # le champs after_payement peux être modifier
        infosups = Info_Sup.objects.filter(ticket=ticket, **infoform)
        self.assertEqual(len(infosups), 1)

    @tag("fast", "local")
    def test_editInfoSupChoice(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # post form
        self.infoSup["type_info"] = Info_Sup.INFO_CHOICE
        d = self.infoSup.copy()
        d.update(self.infoSupChoice)
        response = self.client.post(
            reverse("infosup-add-ticket", args=[event.short_url, ticket.id, Info_Sup.INFO_CHOICE]), d
        )
        self.assertRedirects(response, reverse("details-ticket", args=[event.short_url, ticket.id]))
        infosups = Info_Sup.objects.filter(ticket=ticket, **self.infoSup)
        self.assertEqual(len(infosups), 1)
        info_choice = infosups.first().get_sup_type()
        self.assertEqual(len(info_choice.choices.all()), 2)
        d["choices-TOTAL_FORMS"] = 3
        d["choices-INITIAL_FORMS"] = 2
        d["choices-0-id"] = (info_choice.choices.get(name="TEST1").pk,)
        d["choices-0-name"] = "TEST1-bis"
        d["choices-0-DELETE"] = False
        d["choices-0-info_sup"] = info_choice.pk
        d["choices-1-id"] = (info_choice.choices.get(name="TEST2").pk,)
        d["choices-1-DELETE"] = True
        d["choices-1-info_sup"] = info_choice.pk
        d["choices-2-name"] = "TEST3"
        response2 = self.client.post(
            reverse("infosup-edit-ticket", args=[event.short_url, ticket.id, info_choice.pk]), d
        )
        self.assertRedirects(response2, reverse("details-ticket", args=[event.short_url, ticket.id]))
        infosups = Info_Sup.objects.filter(ticket=ticket, **self.infoSup)
        self.assertEqual(len(infosups), 1)
        info_choice = infosups.first().get_sup_type()
        self.assertEqual(len(info_choice.choices.all()), 2)
        self.assertTrue(info_choice.choices.filter(name="TEST1-bis").exists())
        self.assertTrue(info_choice.choices.filter(name="TEST3").exists())
        self.assertFalse(info_choice.choices.filter(name="TEST2").exists())

    @tag("fast", "local")
    def test_editInfoSupWithRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        infosup = Info_Sup(ticket=ticket, **self.infoSup)
        infosup.save()
        self.client.force_login(user)
        Registration(owner=user, ticket=ticket).save()
        # get form
        response = self.client.get(reverse("infosup-edit-ticket", args=[event.short_url, ticket.id, infosup.pk]))
        self.assertEqual(response.status_code, 200)
        # post
        infoform = self.infoSup.copy()
        infoform["after_payement"] = True
        response2 = self.client.post(
            reverse("infosup-edit-ticket", args=[event.short_url, ticket.id, infosup.pk]), infoform
        )
        self.assertRedirects(response2, reverse("details-ticket", args=[event.short_url, ticket.id]))
        # le champs after_payement ne peux plus être modifier
        infosups = Info_Sup.objects.filter(ticket=ticket, **self.infoSup)
        self.assertEqual(len(infosups), 1)

    @tag("fast", "local")
    def test_delInfoSup(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        infosup = Info_Sup(ticket=ticket, **self.infoSup)
        infosup.save()
        self.client.force_login(user)
        # post
        response = self.client.post(reverse("infosup-del-ticket", args=[event.short_url, ticket.id, infosup.pk]))
        self.assertRedirects(response, reverse("details-ticket", args=[event.short_url, ticket.id]))
        # l'option n'existe plus
        infosups = Info_Sup.objects.filter(ticket=ticket, **self.infoSup)
        self.assertEqual(len(infosups), 0)

    @tag("fast", "local")
    def test_delInfoSupWithRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        infosup = Info_Sup(ticket=ticket, **self.infoSup)
        infosup.save()
        self.client.force_login(user)
        Registration(owner=user, ticket=ticket).save()
        # post
        response = self.client.post(reverse("infosup-del-ticket", args=[event.short_url, ticket.id, infosup.pk]))
        self.assertRedirects(response, reverse("details-ticket", args=[event.short_url, ticket.id]))
        # l'option existe encore
        infosups = Info_Sup.objects.filter(ticket=ticket, **self.infoSup)
        self.assertEqual(len(infosups), 1)

    @tag("fast", "local")
    def test_stateRegistration(self):
        # test la mise à jour de l'état du billet
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        infosup = Info_Sup(ticket=ticket, **self.infoSup)
        infosup.save()
        # 1
        reg = Registration(owner=user, ticket=ticket)
        reg.save()
        self.assertEqual(reg.info_state, Registration.STATE_CANNOT_PAID)
        # 2
        reg.delete()
        infosup.after_payement = True
        infosup.save()
        reg = Registration(owner=user, ticket=ticket)
        reg.save()
        self.assertEqual(reg.info_state, Registration.STATE_CAN_PAID)
        # 3
        reg.paid = True
        reg.save()
        self.assertNotEqual(reg.info_state, Registration.STATE_CAN_ENTER)


class TestCreateOptionAlone(CleanUp, CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    def setUp(self):
        self.tz = pytz.timezone("Europe/Paris")
        self.opt = {
            "name": "optionAloneTest",
            "price": 10,
            "description": "opt description",
            "admin_only": False,
        }

    @tag("fast", "local")
    def test_createOptionAlone(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # get form
        response = self.client.get(reverse("option-alone-add-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response.status_code, 200)
        # post form
        response2 = self.client.post(reverse("option-alone-add-ticket", args=[event.short_url, ticket.id]), self.opt)
        self.assertRedirects(response2, reverse("details-ticket", args=[event.short_url, ticket.id]))
        opts = Option.objects.filter(ticket=ticket, group_name=None, group=None, **self.opt)
        self.assertEqual(len(opts), 1)
        # verify opt in template
        response3 = self.client.get(reverse("details-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response3.status_code, 200)
        self.assertContains(response3, opts.first().name)

    @tag("fast", "local")
    def test_createOptionAloneWithRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        Registration(owner=user, ticket=ticket).save()
        # get form
        response = self.client.get(reverse("option-alone-add-ticket", args=[event.short_url, ticket.id]))
        self.assertRedirects(response, reverse("details-ticket", args=[event.short_url, ticket.id]))

    @tag("fast", "local")
    def test_editOptionAlone(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        opt = Option(ticket=ticket, group_name=None, group=None, **self.opt)
        opt.save()
        self.client.force_login(user)
        # get form
        response = self.client.get(reverse("option-alone-edit-ticket", args=[event.short_url, ticket.id, opt.pk]))
        self.assertEqual(response.status_code, 200)
        # post
        optform = self.opt.copy()
        optform["admin_only"] = True
        optform["price"] = 11
        response2 = self.client.post(
            reverse("option-alone-edit-ticket", args=[event.short_url, ticket.id, opt.pk]), optform
        )
        self.assertRedirects(response2, reverse("details-ticket", args=[event.short_url, ticket.id]))
        # le champs admin_only et price peuvent être modifier
        opts = Option.objects.filter(ticket=ticket, group_name=None, group=None, **optform)
        self.assertEqual(len(opts), 1)
        opts = Option.objects.filter(ticket=ticket, group_name=None, group=None, **self.opt)
        self.assertEqual(len(opts), 0)

    @tag("fast", "local")
    def test_editOptionAloneWithRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        opt = Option(ticket=ticket, group_name=None, group=None, **self.opt)
        opt.save()
        self.client.force_login(user)
        Registration(owner=user, ticket=ticket).save()
        # get form
        response = self.client.get(reverse("option-alone-edit-ticket", args=[event.short_url, ticket.id, opt.pk]))
        self.assertEqual(response.status_code, 200)
        # post
        optform = self.opt.copy()
        optform["admin_only"] = True
        optform["price"] = 11
        response2 = self.client.post(
            reverse("option-alone-edit-ticket", args=[event.short_url, ticket.id, opt.pk]), optform
        )
        self.assertRedirects(response2, reverse("details-ticket", args=[event.short_url, ticket.id]))
        # le champs admin_only et price ne peuvent pas être modifier
        opts = Option.objects.filter(ticket=ticket, group_name=None, group=None, **optform)
        self.assertEqual(len(opts), 0)
        opts = Option.objects.filter(ticket=ticket, group_name=None, group=None, **self.opt)
        self.assertEqual(len(opts), 1)

    @tag("fast", "local")
    def test_delOptionAlone(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        opt = Option(ticket=ticket, group_name=None, group=None, **self.opt)
        opt.save()
        self.client.force_login(user)
        # post
        response = self.client.post(reverse("option-alone-del-ticket", args=[event.short_url, ticket.id, opt.pk]))
        self.assertRedirects(response, reverse("details-ticket", args=[event.short_url, ticket.id]))
        # l'option n'existe plus
        opts = Option.objects.filter(ticket=ticket, group_name=None, group=None, **self.opt)
        self.assertEqual(len(opts), 0)

    @tag("fast", "local")
    def test_delOptionAloneWithRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        opt = Option(ticket=ticket, group_name=None, group=None, **self.opt)
        opt.save()
        self.client.force_login(user)
        Registration(owner=user, ticket=ticket).save()
        # post
        response = self.client.post(reverse("option-alone-del-ticket", args=[event.short_url, ticket.id, opt.pk]))
        self.assertRedirects(response, reverse("details-ticket", args=[event.short_url, ticket.id]))
        # l'option existe encore
        opts = Option.objects.filter(ticket=ticket, group_name=None, group=None, **self.opt)
        self.assertEqual(len(opts), 1)

    @tag("fast", "local")
    def test_VerifyPriceWithOptionAlone(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        opt = Option(ticket=ticket, group_name=None, group=None, **self.opt)
        opt.save()
        self.client.force_login(user)
        reg = Registration(owner=user, ticket=ticket)
        reg.save()
        reg.options.add(opt)
        reg.save()
        self.assertEqual(reg.get_price_options(), opt.price)
        self.assertEqual(reg.get_real_amount(), opt.price + ticket.price)


class TestCreateOptionGroup(CleanUp, CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    def setUp(self):
        self.tz = pytz.timezone("Europe/Paris")
        self.optform = {
            "group_name": "OptTest",
            "description": "Option description",
            "mandatory": False,
            "form-TOTAL_FORMS": 2,
            "form-INITIAL_FORMS": 0,
            "form-MIN_NUM_FORMS": 1,
            "form-MAX_NUM_FORMS": 1000,
            "form-0-name": "TESTopt1",
            "form-0-price": 10,
            "form-0-admin_only": False,
            "form-1-name": "TESTopt2",
            "form-1-price": 20,
            "form-1-admin_only": False,
        }
        self.optform2 = {
            "group_name": "OptTest2",
            "description": "Option description2",
            "mandatory": False,
            "form-TOTAL_FORMS": 3,
            "form-INITIAL_FORMS": 2,
            "form-MIN_NUM_FORMS": 1,
            "form-MAX_NUM_FORMS": 1000,
            "form-0-name": "TESTopt1-new",
            "form-0-price": 11,
            "form-0-admin_only": True,
            "form-0-id": 0,  # à remplir
            "form-0-DELETE": False,
            "form-1-name": "TESTopt2",
            "form-1-price": 20,
            "form-1-admin_only": False,
            "form-1-id": 0,  # à remplir
            "form-1-DELETE": True,
            "form-2-name": "TESTopt3",
            "form-2-price": 30,
            "form-2-admin_only": True,
        }

    @tag("fast", "local")
    def test_createOptionGroup(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # get form
        response = self.client.get(reverse("option-group-add-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response.status_code, 200)
        # post form
        response2 = self.client.post(
            reverse("option-group-add-ticket", args=[event.short_url, ticket.id]), self.optform
        )
        self.assertRedirects(response2, reverse("details-ticket", args=[event.short_url, ticket.id]))
        opts = Option.objects.filter(
            ticket=ticket,
            group_name=self.optform["group_name"],
            description=self.optform["description"],
            mandatory=self.optform["mandatory"],
        )
        self.assertEqual(len(opts), 2)
        group = opts.first().group
        self.assertEqual(
            len(
                opts.filter(
                    name=self.optform["form-0-name"],
                    price=self.optform["form-0-price"],
                    admin_only=self.optform["form-0-admin_only"],
                    group=group,
                )
            ),
            1,
        )
        self.assertEqual(
            len(
                opts.filter(
                    name=self.optform["form-1-name"],
                    price=self.optform["form-1-price"],
                    admin_only=self.optform["form-1-admin_only"],
                    group=group,
                )
            ),
            1,
        )

        # verify opt in template
        response3 = self.client.get(reverse("details-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response3.status_code, 200)
        for opt in opts:
            self.assertContains(response3, opt.name)
        # verify if in list of tickets
        response3 = self.client.get(reverse("list-tickets", args=[event.short_url]))
        self.assertEqual(response3.status_code, 200)
        for opt in opts:
            self.assertContains(response3, opt.name)

    @tag("fast", "local")
    def test_createOptionGroupMandatory(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # post form
        optform = self.optform.copy()
        optform["mandatory"] = True
        response = self.client.post(reverse("option-group-add-ticket", args=[event.short_url, ticket.id]), optform)
        self.assertRedirects(response, reverse("details-ticket", args=[event.short_url, ticket.id]))
        opts = Option.objects.filter(
            ticket=ticket,
            group_name=optform["group_name"],
            description=optform["description"],
            mandatory=optform["mandatory"],
        )
        self.assertEqual(len(opts), 3)
        group = opts.first().group
        self.assertEqual(
            len(
                opts.filter(
                    name=optform["form-0-name"],
                    price=optform["form-0-price"],
                    admin_only=optform["form-0-admin_only"],
                    group=group,
                )
            ),
            1,
        )
        self.assertEqual(
            len(
                opts.filter(
                    name=optform["form-1-name"],
                    price=optform["form-1-price"],
                    admin_only=optform["form-1-admin_only"],
                    group=group,
                )
            ),
            1,
        )
        self.assertEqual(len(opts.filter(price=0, admin_only=False, group=group)), 1)

    @tag("fast", "local")
    def test_createOptionGroupWithRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        Registration(owner=user, ticket=ticket).save()
        # get form
        response = self.client.get(reverse("option-group-add-ticket", args=[event.short_url, ticket.id]))
        self.assertRedirects(response, reverse("details-ticket", args=[event.short_url, ticket.id]))

    @tag("fast", "local")
    def test_editOptionGroup(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # create_option
        response = self.client.post(reverse("option-group-add-ticket", args=[event.short_url, ticket.id]), self.optform)
        self.assertRedirects(response, reverse("details-ticket", args=[event.short_url, ticket.id]))
        opts = Option.objects.filter(
            ticket=ticket, group_name=self.optform["group_name"], description=self.optform["description"]
        )
        group = opts.first().group
        # get
        response = self.client.get(reverse("option-group-edit-ticket", args=[event.short_url, ticket.id, group]))
        self.assertEqual(response.status_code, 200)
        # post
        optform = self.optform2.copy()
        optform["form-0-id"] = opts.filter(name=self.optform["form-0-name"]).first().pk
        optform["form-1-id"] = opts.filter(name=self.optform["form-1-name"]).first().pk
        response2 = self.client.post(
            reverse("option-group-edit-ticket", args=[event.short_url, ticket.id, group]), optform
        )
        self.assertRedirects(response2, reverse("details-ticket", args=[event.short_url, ticket.id]))
        # vérification des modifications
        # les options ont changés de nom de groupe et de description
        self.assertFalse(
            Option.objects.filter(
                ticket=ticket,
                group_name=self.optform["group_name"],
                description=self.optform["description"],
                group=group,
            ).exists()
        )
        #  2 options dans le groupe
        opts2 = Option.objects.filter(
            ticket=ticket, group_name=optform["group_name"], description=optform["description"], group=group
        )
        self.assertEqual(len(opts2), 2)
        # Option 0 existe avec nom, prix et admin modifiés
        self.assertTrue(
            opts2.filter(
                pk=optform["form-0-id"],
                name=optform["form-0-name"],
                price=optform["form-0-price"],
                admin_only=optform["form-0-admin_only"],
            ).exists()
        )
        # Option 1 n'existe plus
        self.assertFalse(Option.objects.filter(pk=optform["form-1-id"]).exists())
        # Option 2 a été crée
        self.assertTrue(
            opts2.filter(
                name=optform["form-2-name"], price=optform["form-2-price"], admin_only=optform["form-2-admin_only"]
            ).exists()
        )

    @tag("fast", "local")
    def test_editOptionGroupWithRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # create_option
        response = self.client.post(reverse("option-group-add-ticket", args=[event.short_url, ticket.id]), self.optform)
        self.assertRedirects(response, reverse("details-ticket", args=[event.short_url, ticket.id]))
        opts = Option.objects.filter(
            ticket=ticket, group_name=self.optform["group_name"], description=self.optform["description"]
        )
        group = opts.first().group
        # create registration
        Registration(owner=user, ticket=ticket).save()
        # get
        response = self.client.get(reverse("option-group-edit-ticket", args=[event.short_url, ticket.id, group]))
        self.assertEqual(response.status_code, 200)
        # post
        optform = self.optform2.copy()
        optform["form-0-id"] = opts.filter(name=self.optform["form-0-name"]).first().pk
        optform["form-1-id"] = opts.filter(name=self.optform["form-1-name"]).first().pk
        response2 = self.client.post(
            reverse("option-group-edit-ticket", args=[event.short_url, ticket.id, group]), optform
        )
        self.assertRedirects(response2, reverse("details-ticket", args=[event.short_url, ticket.id]))
        # vérification des modifications
        # les options ont changés de nom de groupe et de description
        self.assertFalse(
            Option.objects.filter(
                ticket=ticket,
                group_name=self.optform["group_name"],
                description=self.optform["description"],
                group=group,
            ).exists()
        )
        #  2 options dans le groupe
        opts2 = Option.objects.filter(
            ticket=ticket, group_name=optform["group_name"], description=optform["description"], group=group
        )
        self.assertEqual(len(opts2), 2)
        # Option 0 existe avec seul le nom modifié
        self.assertTrue(
            opts2.filter(
                pk=optform["form-0-id"],
                name=optform["form-0-name"],
                price=self.optform["form-0-price"],
                admin_only=self.optform["form-0-admin_only"],
            ).exists()
        )
        # Option 1 existe encore
        self.assertTrue(
            opts2.filter(
                pk=optform["form-1-id"],
                name=optform["form-1-name"],
                price=self.optform["form-1-price"],
                admin_only=self.optform["form-1-admin_only"],
            ).exists()
        )
        # Option 2 n'a pas été créée
        self.assertFalse(opts2.filter(name=optform["form-2-name"]).exists())

    @tag("fast", "local")
    def test_delOptionGroup(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # create_option
        response = self.client.post(reverse("option-group-add-ticket", args=[event.short_url, ticket.id]), self.optform)
        self.assertRedirects(response, reverse("details-ticket", args=[event.short_url, ticket.id]))
        opts = Option.objects.filter(
            ticket=ticket, group_name=self.optform["group_name"], description=self.optform["description"]
        )
        group = opts.first().group
        # post
        response = self.client.post(reverse("option-group-del-ticket", args=[event.short_url, ticket.id, group]))
        self.assertRedirects(response, reverse("details-ticket", args=[event.short_url, ticket.id]))
        # le groupe d'option n'existe plus
        opts = Option.objects.filter(ticket=ticket, group=group)
        self.assertEqual(len(opts), 0)

    @tag("fast", "local")
    def test_delOptionGroupWithRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # create_option
        response = self.client.post(reverse("option-group-add-ticket", args=[event.short_url, ticket.id]), self.optform)
        self.assertRedirects(response, reverse("details-ticket", args=[event.short_url, ticket.id]))
        opts = Option.objects.filter(
            ticket=ticket, group_name=self.optform["group_name"], description=self.optform["description"]
        )
        group = opts.first().group
        # create registration
        Registration(owner=user, ticket=ticket).save()
        # post
        response = self.client.post(reverse("option-group-del-ticket", args=[event.short_url, ticket.id, group]))
        self.assertRedirects(response, reverse("details-ticket", args=[event.short_url, ticket.id]))
        # le groupe d'option existe encore et non modifié
        opts = Option.objects.filter(
            ticket=ticket, group=group, group_name=self.optform["group_name"], description=self.optform["description"]
        )
        self.assertEqual(len(opts), 2)
        self.assertEqual(
            len(
                opts.filter(
                    name=self.optform["form-0-name"],
                    price=self.optform["form-0-price"],
                    admin_only=self.optform["form-0-admin_only"],
                )
            ),
            1,
        )
        self.assertEqual(
            len(
                opts.filter(
                    name=self.optform["form-1-name"],
                    price=self.optform["form-1-price"],
                    admin_only=self.optform["form-1-admin_only"],
                )
            ),
            1,
        )

    @tag("fast", "local")
    def test_VerifyPriceWithOptionGroup(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        # create_option
        response = self.client.post(reverse("option-group-add-ticket", args=[event.short_url, ticket.id]), self.optform)
        self.assertRedirects(response, reverse("details-ticket", args=[event.short_url, ticket.id]))
        opts = Option.objects.filter(
            ticket=ticket, group_name=self.optform["group_name"], description=self.optform["description"]
        )
        for opt in opts:
            reg = Registration(owner=user, ticket=ticket)
            reg.save()
            reg.options.add(opt)
            reg.save()
            self.assertEqual(reg.get_price_options(), opt.price)
            self.assertEqual(reg.get_real_amount(), opt.price + ticket.price)
            reg.delete()


class TestTemplate(CleanUp, CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    def commit_template(self, ticket):
        # on ne supporte que le ticket sans restriction
        self.assertTrue(ticket.allowed_external)
        path = os.path.join(settings.BASE_DIR, "guichet", "tests", "ressources", "ticket_template.png")
        with open(path, "rb") as f:
            img = BytesIO(f.read())
            img.name = "template_test.png"
        data = {
            "name": ticket.name,
            "short_notes": ticket.short_notes,
            "long_notes": ticket.long_notes,
            "price": ticket.price,
            "online_payment": ticket.online_payment,
            "limit": ticket.limit,
            "admin_ticket": ticket.admin_ticket,
            "allow_anonyme": ticket.allow_anonyme,
            "design": img,
        }
        response = self.client.post(reverse("edit-ticket", args=[ticket.event.short_url, ticket.id]), data)
        self.assertRedirects(response, reverse("details-ticket", args=[ticket.event.short_url, ticket.id]))
        self.assertTrue(Ticket.objects.get(pk=ticket.pk).design)
        return Ticket.objects.get(pk=ticket.pk)

    def write_pdf(self, response, name):
        path = os.path.join(settings.BASE_DIR, "guichet", "tests", "result_pdf", name)
        with open(path, "wb") as f:
            f.write(response.content)

    @tag("fast", "local")
    def test_printTestWithoutTemplate(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        event.start_date = timezone.now() + datetime.timedelta(hours=1)
        event.save()
        response = self.client.get(reverse("ticket_test_template", args=[event.short_url, ticket.id]))
        self.assertRedirects(response, reverse("list-tickets", args=[event.short_url]))

    @tag("fast", "local")
    def test_printTestAfterEventBegin(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        self.commit_template(ticket)
        event.start_date = timezone.now() - datetime.timedelta(hours=1)
        event.save()
        response = self.client.get(reverse("ticket_test_template", args=[event.short_url, ticket.id]))
        self.assertRedirects(response, reverse("list-tickets", args=[event.short_url]))

    @tag("low", "local")
    def test_printTest(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        self.commit_template(ticket)
        event.start_date = timezone.now() + datetime.timedelta(hours=1)
        event.end_date = timezone.now() + datetime.timedelta(hours=2)
        event.save()
        response = self.client.get(reverse("ticket_test_template", args=[event.short_url, ticket.id]))
        self.assertEqual(response.status_code, 200)
        self.write_pdf(response, "test_printTest.pdf")

    @tag("fast", "local")
    def test_printRegistrationWithoutTemplate(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        reg = Registration(owner=user, ticket=ticket, paid=True)
        reg.save()
        response = self.client.get(reverse("print-registration", args=[reg.pk, "PDF"]))
        self.assertEqual(response.status_code, 404)

    @tag("fast", "local")
    def test_printRegistrationNoAdminOrOwner(self):
        user = User.objects.get(login_inp_net="testone")
        user2 = User.objects.get(login_inp_net="testwo")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        self.commit_template(ticket)
        self.client.force_login(user2)
        reg = Registration(owner=user, ticket=ticket, paid=True)
        reg.save()
        response = self.client.get(reverse("print-registration", args=[reg.pk, "PDF"]))
        self.assertEqual(response.status_code, 403)

    @tag("fast", "local")
    def test_printRegistrationNotPaid(self):
        user = User.objects.get(login_inp_net="testone")
        user2 = User.objects.get(login_inp_net="testwo")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        self.commit_template(ticket)
        self.client.force_login(user2)
        reg = Registration(owner=user, ticket=ticket, paid=False)
        reg.save()
        response = self.client.get(reverse("print-registration", args=[reg.pk, "PDF"]))
        self.assertEqual(response.status_code, 403)

    @tag("low", "local")
    def test_printRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        self.commit_template(ticket)
        reg = Registration(owner=user, ticket=ticket, paid=True)
        reg.save()
        response = self.client.get(reverse("print-registration", args=[reg.pk, "PDF"]))
        self.assertEqual(response.status_code, 200)
        self.write_pdf(response, "test_printRegistration.pdf")

    @tag("low", "local")
    def test_sendRegistrationByMail(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        self.commit_template(ticket)
        reg = Registration(owner=user, ticket=ticket, paid=True)
        reg.save()
        self.assertEqual(len(mail.outbox), 1)
        response = self.client.get(reverse("mail-registration", args=[reg.pk]))
        self.assertRedirects(response, reverse("show-registration", args=[reg.id]))
        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[0].from_email, settings.DEFAULT_FROM_EMAIL)
        self.assertIn(user.email, mail.outbox[0].to)
        self.assertEqual(mail.outbox[1].from_email, settings.DEFAULT_FROM_EMAIL)
        self.assertIn(user.email, mail.outbox[1].to)

    @tag("low", "local")
    def test_sendRegistrationByMailAdmin(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        self.commit_template(ticket)
        reg = Registration(owner=user, ticket=ticket, paid=True)
        reg.save()
        self.assertEqual(len(mail.outbox), 1)
        response = self.client.post(reverse("email-registration", args=[event.id, reg.id]), {"email": user.email})
        self.assertRedirects(response, reverse("registration-edit", args=[event.id, reg.id]))
        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[0].from_email, settings.DEFAULT_FROM_EMAIL)
        self.assertIn(user.email, mail.outbox[0].to)
        self.assertEqual(mail.outbox[1].from_email, settings.DEFAULT_FROM_EMAIL)
        self.assertIn(user.email, mail.outbox[1].to)

    @tag("fast", "local")
    def test_sendRegistrationByMailInvalidUser(self):
        user = User.objects.get(login_inp_net="testone")
        user2 = User.objects.get(login_inp_net="testwo")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        self.commit_template(ticket)
        self.client.force_login(user2)
        reg = Registration(owner=user, ticket=ticket, paid=True)
        reg.save()
        self.assertEqual(len(mail.outbox), 1)
        response = self.client.get(reverse("mail-registration", args=[reg.pk]))
        self.assertEqual(response.status_code, 403)
        self.assertEqual(len(mail.outbox), 1)

    @tag("fast", "local")
    def test_sendRegistrationByMailNotPaid(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        self.commit_template(ticket)
        reg = Registration(owner=user, ticket=ticket, paid=False)
        reg.save()
        self.assertEqual(len(mail.outbox), 0)
        response = self.client.get(reverse("mail-registration", args=[reg.pk]))
        self.assertEqual(response.status_code, 403)
        self.assertEqual(len(mail.outbox), 0)

    @tag("fast", "local")
    def test_accessTemplate(self):
        user = User.objects.get(login_inp_net="testone")
        user2 = User.objects.get(login_inp_net="testwo")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        ticket = self.commit_template(ticket)
        response = self.client.get(reverse("show-template-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response.status_code, 200)
        response2 = self.client.get(ticket.design.url)
        self.assertRedirects(
            response2,
            reverse("show-template-ticket", args=[event.short_url, ticket.id]) + os.path.basename(ticket.design.path),
        )
        self.client.logout()
        response3 = self.client.get(reverse("show-template-ticket", args=[event.short_url, ticket.id]))
        self.assertRedirects(
            response3, reverse("login") + "?next=" + reverse("show-template-ticket", args=[event.short_url, ticket.id])
        )
        response4 = self.client.get(ticket.design.url)
        self.assertRedirects(response4, reverse("login") + "?next=" + ticket.design.url)
        self.client.force_login(user2)
        response5 = self.client.get(reverse("show-template-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response5.status_code, 403)
        response6 = self.client.get(ticket.design.url)
        self.assertRedirects(
            response6,
            reverse("show-template-ticket", args=[event.short_url, ticket.id]) + os.path.basename(ticket.design.path),
            fetch_redirect_response=False,
        )

    @tag("fast", "local")
    def test_editFormWithDesign(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        ticket = event.tickets.get(id=1)
        self.client.force_login(user)
        ticket = self.commit_template(ticket)
        response = self.client.get(reverse("edit-ticket", args=[event.short_url, ticket.id]))
        self.assertEqual(response.status_code, 200)


class TestLogs(CleanUp, CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    @tag("fast", "local")
    def test_accesslogs(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        response = self.client.get(reverse("show-logs", args=[event.url()]))
        self.assertEqual(response.status_code, 200)


class TestStat(CleanUp, CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    def setUp(self):
        self.event = Event.objects.get(pk=1)
        self.restrictedticket = Ticket(
            event=self.event,
            name="RestrictedTicket",
            id=Ticket.next_id(self.event),
            short_notes="short description",
            long_notes="long description",
            price=10.0,
            online_payment=False,
            allowed_external=False,
            limit=0,
            required_school=Ticket.SCHOOL_INP,
            required_ae=False,
            allowed_godson=False,
            admin_ticket=False,
            allow_anonyme=False,
        )
        self.restrictedticket.save()
        # options alone
        self.optalone1 = Option(
            name="OptAlone1",
            price=5,
            description="Alone is alone",
            group_name=None,
            group=None,
            ticket=self.restrictedticket,
            admin_only=False,
        )
        self.optalone1.save()
        # options group
        self.optgrp1c1 = Option(
            name="Choice1",
            price=15,
            description="Group is grouped",
            group_name="GroupOPT1",
            group=1,
            ticket=self.restrictedticket,
            admin_only=False,
        )
        self.optgrp1c1.save()
        self.optgrp1c2 = Option(
            name="Choice2",
            price=10,
            description="Group is grouped",
            group_name="GroupOPT1",
            group=1,
            ticket=self.restrictedticket,
            admin_only=False,
        )
        self.optgrp1c2.save()

    @tag("fast", "local")
    def test_accesslogs(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        response = self.client.get(reverse("show-stats", args=[event.url()]))
        self.assertEqual(response.status_code, 200)


class TestManagerPage(CleanUp, CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    def test_getpage(self):
        user = User.objects.get(login_inp_net="testone")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        response = self.client.get(reverse("list-managers", args=[event.short_url]))
        self.assertEqual(response.status_code, 200)

    @tag("fast", "local")
    def test_promoteUser(self):
        user = User.objects.get(login_inp_net="testone")
        user2 = User.objects.get(login_inp_net="testwo")
        user3 = User.objects.get(email="exte@billetterie.inpt.fr")
        user4 = User.get_anonyme_user()
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        # promote INP-net
        response = self.client.post(
            reverse("list-managers", args=[event.short_url]),
            {"promote-user": "", "login": user2.login_inp_net, "access": Manager.ENTRY_MANAGER},
        )
        self.assertRedirects(response, reverse("list-managers", args=[event.short_url]))
        self.assertTrue(UserManager.objects.filter(event=event, user=user2, access=Manager.ENTRY_MANAGER).exists())
        self.assertEqual(len(UserManager.objects.filter(event=event, user=user2)), 1)
        # promote Exte
        response2 = self.client.post(
            reverse("list-managers", args=[event.short_url]),
            {"promote-user": "", "email": user3.email, "access": Manager.FULL_VIEW},
        )
        self.assertRedirects(response2, reverse("list-managers", args=[event.short_url]))
        self.assertTrue(UserManager.objects.filter(event=event, user=user3, access=Manager.FULL_VIEW).exists())
        self.assertEqual(len(UserManager.objects.filter(event=event, user=user3)), 1)
        # promote Anonyme
        response3 = self.client.post(
            reverse("list-managers", args=[event.short_url]),
            {"promote-user": "", "email": user4.email, "access": Manager.FULL_VIEW},
        )
        self.assertEqual(response3.status_code, 200)
        self.assertEqual(len(UserManager.objects.filter(event=event, user=user4)), 0)

    @tag("fast", "local")
    def test_promoteGroup(self):
        user = User.objects.get(login_inp_net="testone")
        group = Group.objects.get(pk=1, cn="test-inp")
        event = Event.objects.get(pk=1)
        self.client.force_login(user)
        # promote Club
        response = self.client.post(
            reverse("list-managers", args=[event.short_url]),
            {"promote-group": "", "group": group.pk, "access": Manager.FULL_VIEW},
        )
        self.assertRedirects(response, reverse("list-managers", args=[event.short_url]))
        self.assertTrue(
            GroupManager.objects.filter(event=event, group=group, bureau_only=False, access=Manager.FULL_VIEW).exists()
        )
        # promote Bureau
        response2 = self.client.post(
            reverse("list-managers", args=[event.short_url]),
            {"promote-group": "", "group": group.pk, "bureau_only": True, "access": Manager.ENTRY_MANAGER},
        )
        self.assertRedirects(response2, reverse("list-managers", args=[event.short_url]))
        self.assertTrue(
            GroupManager.objects.filter(
                event=event, group=group, bureau_only=True, access=Manager.ENTRY_MANAGER
            ).exists()
        )

    @tag("fast", "local")
    def test_editUser(self):
        event = Event.objects.get(pk=1)
        user = User.objects.get(login_inp_net="testone")
        user2 = User.objects.get(login_inp_net="testwo")
        user3 = User.objects.get(email="exte@billetterie.inpt.fr")
        perm2 = UserManager(event=event, user=user2, access=Manager.ENTRY_MANAGER)
        perm2.save()
        perm3 = UserManager(event=event, user=user3, access=Manager.FULL_VIEW)
        perm3.save()
        self.client.force_login(user)
        # get and promote same permission
        response = self.client.get(reverse("edit-manager-user", args=[event.short_url, perm2.pk]))
        self.assertEqual(response.status_code, 200)
        response2 = self.client.post(
            reverse("edit-manager-user", args=[event.short_url, perm2.pk]),
            {"promote-user": "", "login": user2.login_inp_net, "access": Manager.ENTRY_MANAGER},
        )
        self.assertRedirects(response2, reverse("list-managers", args=[event.short_url]))
        self.assertTrue(UserManager.objects.filter(event=event, user=user2, access=Manager.ENTRY_MANAGER).exists())
        self.assertEqual(len(UserManager.objects.filter(event=event, user=user2)), 1)
        # get and promote same permission
        response3 = self.client.get(reverse("edit-manager-user", args=[event.short_url, perm3.pk]))
        self.assertEqual(response3.status_code, 200)
        response4 = self.client.post(
            reverse("edit-manager-user", args=[event.short_url, perm3.pk]),
            {"promote-user": "", "email": user3.email, "access": Manager.ENTRY_MANAGER},
        )
        self.assertRedirects(response4, reverse("list-managers", args=[event.short_url]))
        self.assertTrue(UserManager.objects.filter(event=event, user=user3, access=Manager.ENTRY_MANAGER).exists())
        self.assertEqual(len(UserManager.objects.filter(event=event, user=user3)), 1)

    @tag("fast", "local")
    def test_editGroup(self):
        user = User.objects.get(login_inp_net="testone")
        group = Group.objects.get(pk=1, cn="test-inp")
        event = Event.objects.get(pk=1)
        perm_club = GroupManager(event=event, group=group, bureau_only=False, access=Manager.ENTRY_MANAGER)
        perm_club.save()
        perm_bureau = GroupManager(event=event, group=group, bureau_only=True, access=Manager.ENTRY_MANAGER)
        perm_bureau.save()
        self.client.force_login(user)
        # get Club and change permission
        response = self.client.get(reverse("edit-manager-group", args=[event.short_url, perm_club.pk]))
        self.assertEqual(response.status_code, 200)
        response2 = self.client.post(
            reverse("edit-manager-group", args=[event.short_url, perm_club.pk]),
            {"promote-group": "", "group": group.pk, "access": Manager.FULL_VIEW},
        )
        self.assertRedirects(response2, reverse("list-managers", args=[event.short_url]))
        self.assertTrue(
            GroupManager.objects.filter(event=event, group=group, bureau_only=False, access=Manager.FULL_VIEW).exists()
        )
        self.assertEqual(len(GroupManager.objects.filter(event=event, group=group, bureau_only=False)), 1)
        # get Bureau and reasign same permission
        response = self.client.get(reverse("edit-manager-group", args=[event.short_url, perm_bureau.pk]))
        self.assertEqual(response.status_code, 200)
        response4 = self.client.post(
            reverse("edit-manager-group", args=[event.short_url, perm_bureau.pk]),
            {"promote-group": "", "group": group.pk, "bureau_only": True, "access": Manager.ENTRY_MANAGER},
        )
        self.assertRedirects(response4, reverse("list-managers", args=[event.short_url]))
        self.assertTrue(
            GroupManager.objects.filter(
                event=event, group=group, bureau_only=True, access=Manager.ENTRY_MANAGER
            ).exists()
        )
        self.assertEqual(len(GroupManager.objects.filter(event=event, group=group, bureau_only=True)), 1)

    @tag("fast", "local")
    def test_deleteUser(self):
        event = Event.objects.get(pk=1)
        user = User.objects.get(login_inp_net="testone")
        user2 = User.objects.get(login_inp_net="testwo")
        perm2 = UserManager(event=event, user=user2, access=Manager.ENTRY_MANAGER)
        perm2.save()
        self.client.force_login(user)
        response = self.client.post(reverse("revoke-manager-user", args=[event.short_url, perm2.pk]))
        self.assertRedirects(response, reverse("list-managers", args=[event.short_url]))
        self.assertFalse(UserManager.objects.filter(event=event, user=user2).exists())

    @tag("fast", "local")
    def test_deleteGroup(self):
        event = Event.objects.get(pk=1)
        user = User.objects.get(login_inp_net="testone")
        group = Group.objects.get(pk=1, cn="test-inp")
        event = Event.objects.get(pk=1)
        perm_club = GroupManager(event=event, group=group, bureau_only=False, access=Manager.ENTRY_MANAGER)
        perm_club.save()
        perm_bureau = GroupManager(event=event, group=group, bureau_only=True, access=Manager.ENTRY_MANAGER)
        perm_bureau.save()
        self.client.force_login(user)
        response = self.client.post(reverse("revoke-manager-group", args=[event.short_url, perm_club.pk]))
        self.assertRedirects(response, reverse("list-managers", args=[event.short_url]))
        self.assertFalse(GroupManager.objects.filter(event=event, group=group, bureau_only=False).exists())
        response2 = self.client.post(reverse("revoke-manager-group", args=[event.short_url, perm_bureau.pk]))
        self.assertRedirects(response2, reverse("list-managers", args=[event.short_url]))
        self.assertFalse(GroupManager.objects.filter(event=event, group=group, bureau_only=True).exists())
        self.assertFalse(GroupManager.objects.filter(event=event, group=group).exists())


class TestEnterInfoSup(CleanUp, CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    def setUp(self):
        self.tz = pytz.timezone("Europe/Paris")
        self.now = timezone.now().astimezone(self.tz)
        self.event = Event.objects.get(pk=1)
        self.event.tickets.all().delete()
        self.event.start_date = (self.now + datetime.timedelta(days=1)).astimezone(self.tz)
        self.event.end_date = (self.now + datetime.timedelta(days=2)).astimezone(self.tz)
        self.event.save()
        self.ticket = Ticket(
            event=self.event,
            name="OpenTicket",
            id=Ticket.next_id(self.event),
            short_notes="short description",
            long_notes="long description",
            price=10.0,
            online_payment=False,
            allowed_external=True,
            limit=0,
            admin_ticket=False,
            allow_anonyme=False,
        )
        self.ticket.save()
        # info Sup
        self.infosimple = Info_Sup(
            ticket=self.ticket,
            name="InfoSimple",
            description="Description Simple",
            after_payement=False,
            type_info=Info_Sup.INFO_SIMPLE,
            info_validate=False,
            info_editable=False,
        )
        self.infoshorttext = Info_Sup(
            ticket=self.ticket,
            name="InfoShortText",
            description="Description Short Text",
            after_payement=False,
            type_info=Info_Sup.INFO_SHORT_TEXT,
            info_validate=False,
            info_editable=False,
        )
        self.infolongtext = Info_Sup(
            ticket=self.ticket,
            name="InfoLongText",
            description="Description Long Text",
            after_payement=False,
            type_info=Info_Sup.INFO_LONG_TEXT,
            info_validate=False,
            info_editable=False,
        )
        self.infochoices = Info_Sup_Choice(
            ticket=self.ticket,
            name="InfoChoice",
            description="Description Choice",
            after_payement=False,
            type_info=Info_Sup.INFO_CHOICE,
            info_validate=False,
            info_editable=False,
        )
        self.choice1 = Info_Sup_Choice_Choices(
            info_sup=self.infochoices,
            name="Choice 1",
        )
        self.choice2 = Info_Sup_Choice_Choices(
            info_sup=self.infochoices,
            name="Choice 2",
        )
        self.infoint = Info_Sup_Int(
            ticket=self.ticket,
            name="InfoInt",
            description="Description Int",
            after_payement=False,
            type_info=Info_Sup.INFO_INT,
            info_validate=False,
            info_editable=False,
            min_value=None,
            max_value=None,
        )
        self.infofloat = Info_Sup_Float(
            ticket=self.ticket,
            name="InfoFloat",
            description="Description Float",
            after_payement=False,
            type_info=Info_Sup.INFO_FLOAT,
            info_validate=False,
            info_editable=False,
            min_value=None,
            max_value=None,
        )
        self.infofile = Info_Sup_File(
            ticket=self.ticket,
            name="InfoFile",
            description="Description File",
            after_payement=False,
            type_info=Info_Sup.INFO_FILE,
            info_validate=False,
            info_editable=False,
            file_name="file_name",
        )
        # User
        self.user4 = User.objects.get(login_inp_net="testfour")
        self.user2 = User.objects.get(login_inp_net="testwo")
        self.useradmin = User.objects.get(login_inp_net="testone")
        self.usersuperadmin = User.objects.get(email="admin@billetterie.inpt.fr")
        # Registration
        self.reg = Registration(
            ticket=self.ticket,
            owner=self.user4,
            payee=None,
            paid=False,
        )
        # file for test
        self.path_img = os.path.join(settings.BASE_DIR, "guichet", "tests", "ressources", "ticket_template.png")
        self.path_pdf = os.path.join(settings.BASE_DIR, "guichet", "tests", "ressources", "logo_net7.pdf")

    def save_all(self):
        self.infosimple.save()
        self.infoshorttext.save()
        self.infolongtext.save()
        self.infochoices.save()
        self.choice1.info_sup = self.infochoices
        self.choice2.info_sup = self.infochoices
        self.choice1.save()
        self.choice2.save()
        self.infoint.save()
        self.infofloat.save()
        self.infofloat.save()
        self.reg.save()

    def get_user_form(self, reg):
        c = Client()
        c.force_login(self.user4)
        response = c.get(reverse("show-registration", args=[reg.id]))
        self.assertEqual(response.status_code, 200)
        context = response.context
        self.assertIn("info_associated_form", context)
        data = {}
        for info, form in context["info_associated_form"]:
            for key, field in form.fields.items():
                data[key] = field.initial
        return data

    def get_admin_form(self, reg):
        c = Client()
        c.force_login(self.usersuperadmin)
        response = c.get(reverse("registration-edit", args=[reg.ticket.event.short_url, reg.id]))
        self.assertEqual(response.status_code, 200)
        context = response.context
        self.assertIn("info_sup_form", context)
        data = {}
        for info, form in context["info_sup_form"]:
            for key, field in form.fields.items():
                data[key] = field.initial
        return data

    @tag("fast", "local")
    def test_InfoSimpleUserForm(self):
        self.infosimple.save()
        self.reg.save()
        self.client.force_login(self.user4)
        form = self.get_user_form(self.reg)
        recv_infosimple = self.infosimple.received.get(registration=self.reg)
        self.assertEqual(len(form), 1)
        self.assertIn("info-{}-uuid".format(recv_infosimple.pk), form)
        self.assertEqual(recv_infosimple.version, form["info-{}-uuid".format(recv_infosimple.pk)])
        self.assertNotIn("info-{}-value".format(recv_infosimple.pk), form)
        response = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infosimple.pk)
        self.assertEqual(new_info.version, recv_infosimple.version)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_MISSING)

    @tag("fast", "local")
    def test_InfoShortTextUserForm(self):
        self.infoshorttext.save()
        self.reg.save()
        self.client.force_login(self.user4)
        form = self.get_user_form(self.reg)
        recv_infoshorttext = self.infoshorttext.received.get(registration=self.reg).get_sup_type()
        self.assertEqual(len(form), 2)
        self.assertIn("info-{}-uuid".format(recv_infoshorttext.pk), form)
        self.assertEqual(recv_infoshorttext.version, form["info-{}-uuid".format(recv_infoshorttext.pk)])
        self.assertIn("info-{}-value".format(recv_infoshorttext.pk), form)
        self.assertEqual(recv_infoshorttext.value, form["info-{}-value".format(recv_infoshorttext.pk)])
        # Post no change
        response = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertEqual(new_info.version, recv_infoshorttext.version)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_MISSING)
        # Post Change
        form["info-{}-value".format(recv_infoshorttext.pk)] = "Personnal Data"
        response2 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response2, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertNotEqual(new_info.version, recv_infoshorttext.version)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_PENDING)
        self.assertEqual(new_info.value, "Personnal Data")
        # Post anti raced condition
        form["info-{}-value".format(recv_infoshorttext.pk)] = "Personnal Data 2"
        response3 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response3, reverse("show-registration", args=[self.reg.id]))
        raced_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertEqual(new_info.version, raced_info.version)
        self.assertEqual(raced_info.value, "Personnal Data")
        # Set to Null
        form = self.get_user_form(self.reg)
        form["info-{}-value".format(recv_infoshorttext.pk)] = ""
        response4 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response4, reverse("show-registration", args=[self.reg.id]))
        null_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertNotEqual(new_info.version, null_info.version)
        self.assertEqual(null_info.value, "")
        self.assertEqual(null_info.state, Info_Sup_Recv.STATE_MISSING)

    @tag("fast", "local")
    def test_InfoShortTextAdminForm(self):
        self.infoshorttext.save()
        self.reg.save()
        self.client.force_login(self.usersuperadmin)
        form = self.get_admin_form(self.reg)
        recv_infoshorttext = self.infoshorttext.received.get(registration=self.reg).get_sup_type()
        self.assertEqual(len(form), 3)
        self.assertIn("info-{}-uuid".format(recv_infoshorttext.pk), form)
        self.assertEqual(recv_infoshorttext.version, form["info-{}-uuid".format(recv_infoshorttext.pk)])
        self.assertIn("info-{}-value".format(recv_infoshorttext.pk), form)
        self.assertEqual(recv_infoshorttext.value, form["info-{}-value".format(recv_infoshorttext.pk)])
        self.assertIn("info-{}-state".format(recv_infoshorttext.pk), form)
        self.assertEqual(recv_infoshorttext.state, int(form["info-{}-state".format(recv_infoshorttext.pk)]))
        # Post no change
        response = self.client.post(
            reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id]), form
        )
        self.assertRedirects(
            response, reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id])
        )
        new_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertEqual(new_info.version, recv_infoshorttext.version)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_MISSING)
        # Post Change
        form["info-{}-value".format(recv_infoshorttext.pk)] = "Personnal Data"
        response2 = self.client.post(
            reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id]), form
        )
        self.assertRedirects(
            response2, reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id])
        )
        new_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertNotEqual(new_info.version, recv_infoshorttext.version)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_MISSING)
        self.assertEqual(new_info.value, "Personnal Data")
        # Post anti raced condition
        form["info-{}-value".format(recv_infoshorttext.pk)] = "Personnal Data 2"
        response3 = self.client.post(
            reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id]), form
        )
        self.assertRedirects(
            response3, reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id])
        )
        raced_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertEqual(new_info.version, raced_info.version)
        self.assertEqual(raced_info.value, "Personnal Data")
        # Set to Null
        form = self.get_admin_form(self.reg)
        form["info-{}-value".format(recv_infoshorttext.pk)] = ""
        response4 = self.client.post(
            reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id]), form
        )
        self.assertRedirects(
            response4, reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id])
        )
        null_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertNotEqual(new_info.version, null_info.version)
        self.assertEqual(null_info.value, "")
        self.assertEqual(null_info.state, Info_Sup_Recv.STATE_MISSING)
        # set info to Reject
        form = self.get_admin_form(self.reg)
        form["info-{}-value".format(recv_infoshorttext.pk)] = "Personnal Data"
        form["info-{}-state".format(recv_infoshorttext.pk)] = Info_Sup_Recv.STATE_REJECTED
        response5 = self.client.post(
            reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id]), form
        )
        self.assertRedirects(
            response5, reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id])
        )
        null_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertNotEqual(new_info.version, null_info.version)
        self.assertEqual(null_info.state, Info_Sup_Recv.STATE_REJECTED)
        # set info to Pending
        form = self.get_admin_form(self.reg)
        form["info-{}-state".format(recv_infoshorttext.pk)] = Info_Sup_Recv.STATE_PENDING
        response6 = self.client.post(
            reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id]), form
        )
        self.assertRedirects(
            response6, reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id])
        )
        null_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertNotEqual(new_info.version, null_info.version)
        self.assertEqual(null_info.state, Info_Sup_Recv.STATE_PENDING)
        # set info to missing
        form = self.get_admin_form(self.reg)
        form["info-{}-state".format(recv_infoshorttext.pk)] = Info_Sup_Recv.STATE_MISSING
        response7 = self.client.post(
            reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id]), form
        )
        self.assertRedirects(
            response7, reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id])
        )
        null_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertNotEqual(new_info.version, null_info.version)
        self.assertEqual(null_info.state, Info_Sup_Recv.STATE_MISSING)
        # set info to Valid
        form = self.get_admin_form(self.reg)
        form["info-{}-state".format(recv_infoshorttext.pk)] = Info_Sup_Recv.STATE_ACCEPTED
        response8 = self.client.post(
            reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id]), form
        )
        self.assertRedirects(
            response8, reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id])
        )
        null_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertNotEqual(new_info.version, null_info.version)
        self.assertEqual(null_info.state, Info_Sup_Recv.STATE_ACCEPTED)
        # try invalid Valid Info
        form = self.get_admin_form(self.reg)
        form["info-{}-state".format(recv_infoshorttext.pk)] = Info_Sup_Recv.STATE_PENDING
        response9 = self.client.post(
            reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id]), form
        )
        self.assertRedirects(
            response9, reverse("registration-edit", args=[self.reg.ticket.event.short_url, self.reg.id])
        )
        null_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertNotEqual(new_info.version, null_info.version)
        self.assertEqual(null_info.state, Info_Sup_Recv.STATE_ACCEPTED)

    @tag("fast", "local")
    def test_InfoLongTextUserForm(self):
        self.infolongtext.save()
        self.reg.save()
        self.client.force_login(self.user4)
        form = self.get_user_form(self.reg)
        recv_infolongtext = self.infolongtext.received.get(registration=self.reg).get_sup_type()
        self.assertEqual(len(form), 2)
        self.assertIn("info-{}-uuid".format(recv_infolongtext.pk), form)
        self.assertEqual(recv_infolongtext.version, form["info-{}-uuid".format(recv_infolongtext.pk)])
        self.assertIn("info-{}-value".format(recv_infolongtext.pk), form)
        self.assertEqual(recv_infolongtext.value, form["info-{}-value".format(recv_infolongtext.pk)])
        # Post no change
        response = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infolongtext.pk)
        self.assertEqual(new_info.version, recv_infolongtext.version)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_MISSING)
        # Post Change
        form["info-{}-value".format(recv_infolongtext.pk)] = "Personnal Data"
        response2 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response2, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infolongtext.pk)
        self.assertNotEqual(new_info.version, recv_infolongtext.version)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_PENDING)
        self.assertEqual(new_info.value, "Personnal Data")
        # Post anti raced condition
        form["info-{}-value".format(recv_infolongtext.pk)] = "Personnal Data 2"
        response3 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response3, reverse("show-registration", args=[self.reg.id]))
        raced_info = Info_Sup_Recv.objects.get(pk=recv_infolongtext.pk)
        self.assertEqual(new_info.version, raced_info.version)
        self.assertEqual(raced_info.value, "Personnal Data")
        # Set to Null
        form = self.get_user_form(self.reg)
        form["info-{}-value".format(recv_infolongtext.pk)] = ""
        response4 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response4, reverse("show-registration", args=[self.reg.id]))
        null_info = Info_Sup_Recv.objects.get(pk=recv_infolongtext.pk)
        self.assertNotEqual(new_info.version, null_info.version)
        self.assertEqual(null_info.value, "")
        self.assertEqual(null_info.state, Info_Sup_Recv.STATE_MISSING)

    @tag("fast", "local")
    def test_InfoChoicesUserForm(self):
        self.infochoices.save()
        self.choice1.info_sup = self.infochoices
        self.choice2.info_sup = self.infochoices
        self.choice1.save()
        self.choice2.save()
        self.reg.save()
        self.client.force_login(self.user4)
        form = self.get_user_form(self.reg)
        recv_infochoices = self.infochoices.received.get(registration=self.reg).get_sup_type()
        self.assertEqual(len(form), 2)
        self.assertIn("info-{}-uuid".format(recv_infochoices.pk), form)
        self.assertEqual(recv_infochoices.version, form["info-{}-uuid".format(recv_infochoices.pk)])
        self.assertIn("info-{}-value".format(recv_infochoices.pk), form)
        self.assertEqual(recv_infochoices.value, form["info-{}-value".format(recv_infochoices.pk)])
        # Post no change
        form["info-{}-value".format(recv_infochoices.pk)] = ""
        response = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infochoices.pk)
        self.assertEqual(new_info.version, recv_infochoices.version)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_MISSING)
        # Post Change
        form["info-{}-value".format(recv_infochoices.pk)] = self.choice1.pk
        response2 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response2, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infochoices.pk)
        self.assertNotEqual(new_info.version, recv_infochoices.version)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_PENDING)
        self.assertEqual(new_info.value, self.choice1)
        # Post anti raced condition
        form["info-{}-value".format(recv_infochoices.pk)] = self.choice2.pk
        response3 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response3, reverse("show-registration", args=[self.reg.id]))
        raced_info = Info_Sup_Recv.objects.get(pk=recv_infochoices.pk)
        self.assertEqual(new_info.version, raced_info.version)
        self.assertEqual(raced_info.value, self.choice1)
        # Set to Null
        form = self.get_user_form(self.reg)
        form["info-{}-value".format(recv_infochoices.pk)] = ""
        response4 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response4, reverse("show-registration", args=[self.reg.id]))
        null_info = Info_Sup_Recv.objects.get(pk=recv_infochoices.pk)
        self.assertNotEqual(new_info.version, null_info.version)
        self.assertEqual(null_info.value, None)
        self.assertEqual(null_info.state, Info_Sup_Recv.STATE_MISSING)

    @tag("fast", "local")
    def test_InfoIntUserForm(self):
        self.infoint.save()
        self.reg.save()
        self.client.force_login(self.user4)
        form = self.get_user_form(self.reg)
        recv_infoint = self.infoint.received.get(registration=self.reg).get_sup_type()
        self.assertEqual(len(form), 2)
        self.assertIn("info-{}-uuid".format(recv_infoint.pk), form)
        self.assertEqual(recv_infoint.version, form["info-{}-uuid".format(recv_infoint.pk)])
        self.assertIn("info-{}-value".format(recv_infoint.pk), form)
        self.assertEqual(recv_infoint.value, form["info-{}-value".format(recv_infoint.pk)])
        # Post no change
        form["info-{}-value".format(recv_infoint.pk)] = ""
        response = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infoint.pk)
        self.assertEqual(new_info.version, recv_infoint.version)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_MISSING)
        # Post Change
        form["info-{}-value".format(recv_infoint.pk)] = 10
        response2 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response2, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infoint.pk)
        self.assertNotEqual(new_info.version, recv_infoint.version)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_PENDING)
        self.assertEqual(new_info.value, 10)
        # Post anti raced condition
        form["info-{}-value".format(recv_infoint.pk)] = 12
        response3 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response3, reverse("show-registration", args=[self.reg.id]))
        raced_info = Info_Sup_Recv.objects.get(pk=recv_infoint.pk)
        self.assertEqual(new_info.version, raced_info.version)
        self.assertEqual(raced_info.value, 10)
        # Set to Null
        form = self.get_user_form(self.reg)
        form["info-{}-value".format(recv_infoint.pk)] = ""
        response4 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response4, reverse("show-registration", args=[self.reg.id]))
        null_info = Info_Sup_Recv.objects.get(pk=recv_infoint.pk)
        self.assertNotEqual(new_info.version, null_info.version)
        self.assertEqual(null_info.value, None)
        self.assertEqual(null_info.state, Info_Sup_Recv.STATE_MISSING)

    @tag("fast", "local")
    def test_InfoIntLimitUserForm(self):
        self.infoint.min_value = 10
        self.infoint.max_value = 20
        self.infoint.save()
        self.reg.save()
        self.client.force_login(self.user4)
        form = self.get_user_form(self.reg)
        recv_infoint = self.infoint.received.get(registration=self.reg).get_sup_type()
        # Post Valid Change Min
        form["info-{}-value".format(recv_infoint.pk)] = 10
        response = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infoint.pk)
        self.assertEqual(new_info.value, 10)
        # Post Invalid Min
        form["info-{}-value".format(recv_infoint.pk)] = 9
        response2 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertEqual(response2.status_code, 200)
        new_info = Info_Sup_Recv.objects.get(pk=recv_infoint.pk)
        self.assertEqual(new_info.value, 10)
        # Post Valid Change Max
        form = self.get_user_form(self.reg)
        form["info-{}-value".format(recv_infoint.pk)] = 20
        response3 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response3, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infoint.pk)
        self.assertEqual(new_info.value, 20)
        # Post Invalid Max
        form = self.get_user_form(self.reg)
        form["info-{}-value".format(recv_infoint.pk)] = 21
        response4 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertEqual(response4.status_code, 200)
        new_info = Info_Sup_Recv.objects.get(pk=recv_infoint.pk)
        self.assertEqual(new_info.value, 20)

    @tag("fast", "local")
    def test_InfoFloatUserForm(self):
        self.infofloat.save()
        self.reg.save()
        self.client.force_login(self.user4)
        form = self.get_user_form(self.reg)
        recv_infofloat = self.infofloat.received.get(registration=self.reg).get_sup_type()
        self.assertEqual(len(form), 2)
        self.assertIn("info-{}-uuid".format(recv_infofloat.pk), form)
        self.assertEqual(recv_infofloat.version, form["info-{}-uuid".format(recv_infofloat.pk)])
        self.assertIn("info-{}-value".format(recv_infofloat.pk), form)
        self.assertEqual(recv_infofloat.value, form["info-{}-value".format(recv_infofloat.pk)])
        # Post no change
        form["info-{}-value".format(recv_infofloat.pk)] = ""
        response = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infofloat.pk)
        self.assertEqual(new_info.version, recv_infofloat.version)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_MISSING)
        # Post Change
        form["info-{}-value".format(recv_infofloat.pk)] = 10.5
        response2 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response2, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infofloat.pk)
        self.assertNotEqual(new_info.version, recv_infofloat.version)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_PENDING)
        self.assertEqual(new_info.value, 10.5)
        # Post anti raced condition
        form["info-{}-value".format(recv_infofloat.pk)] = 12.26
        response3 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response3, reverse("show-registration", args=[self.reg.id]))
        raced_info = Info_Sup_Recv.objects.get(pk=recv_infofloat.pk)
        self.assertEqual(new_info.version, raced_info.version)
        self.assertEqual(raced_info.value, 10.5)
        # Set to Null
        form = self.get_user_form(self.reg)
        form["info-{}-value".format(recv_infofloat.pk)] = ""
        response4 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response4, reverse("show-registration", args=[self.reg.id]))
        null_info = Info_Sup_Recv.objects.get(pk=recv_infofloat.pk)
        self.assertNotEqual(new_info.version, null_info.version)
        self.assertEqual(null_info.value, None)
        self.assertEqual(null_info.state, Info_Sup_Recv.STATE_MISSING)

    @tag("fast", "local")
    def test_InfoFloatLimitUserForm(self):
        self.infofloat.min_value = 10.5
        self.infofloat.max_value = 20.3
        self.infofloat.save()
        self.reg.save()
        self.client.force_login(self.user4)
        form = self.get_user_form(self.reg)
        recv_infofloat = self.infofloat.received.get(registration=self.reg).get_sup_type()
        # Post Valid Change Min
        form["info-{}-value".format(recv_infofloat.pk)] = 10.5
        response = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infofloat.pk)
        self.assertEqual(new_info.value, 10.5)
        # Post Invalid Min
        form["info-{}-value".format(recv_infofloat.pk)] = 10.4
        response2 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertEqual(response2.status_code, 200)
        new_info = Info_Sup_Recv.objects.get(pk=recv_infofloat.pk)
        self.assertEqual(new_info.value, 10.5)
        # Post Valid Change Max
        form = self.get_user_form(self.reg)
        form["info-{}-value".format(recv_infofloat.pk)] = 20.3
        response3 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response3, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infofloat.pk)
        self.assertEqual(new_info.value, 20.3)
        # Post Invalid Max
        form = self.get_user_form(self.reg)
        form["info-{}-value".format(recv_infofloat.pk)] = 20.4
        response4 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertEqual(response4.status_code, 200)
        new_info = Info_Sup_Recv.objects.get(pk=recv_infofloat.pk)
        self.assertEqual(new_info.value, 20.3)

    @tag("low", "local")
    def test_InfoFileUserForm(self):
        self.infofile.save()
        self.reg.save()
        self.client.force_login(self.user4)
        form = self.get_user_form(self.reg)
        recv_infofile = self.infofile.received.get(registration=self.reg).get_sup_type()
        self.assertEqual(len(form), 2)
        self.assertIn("info-{}-uuid".format(recv_infofile.pk), form)
        self.assertEqual(recv_infofile.version, form["info-{}-uuid".format(recv_infofile.pk)])
        self.assertIn("info-{}-value".format(recv_infofile.pk), form)
        self.assertEqual(recv_infofile.value, form["info-{}-value".format(recv_infofile.pk)])
        # Post no change
        form["info-{}-value".format(recv_infofile.pk)] = ""
        response = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infofile.pk)
        self.assertEqual(new_info.version, recv_infofile.version)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_MISSING)
        # Post Change
        with open(self.path_img, "rb") as f:
            content_img = f.read()
            img = BytesIO(content_img)
            img.name = "test.png"
        form["info-{}-value".format(recv_infofile.pk)] = img
        response2 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response2, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infofile.pk)
        self.assertNotEqual(new_info.version, recv_infofile.version)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_PENDING)
        content_file = new_info.value.read()
        self.assertEqual(content_file, content_img)
        # Post anti raced condition
        with open(self.path_pdf, "rb") as f:
            content_pdf = f.read()
            pdf = BytesIO(content_pdf)
            pdf.name = "test.pdf"
        form["info-{}-value".format(recv_infofile.pk)] = pdf
        response3 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response3, reverse("show-registration", args=[self.reg.id]))
        raced_info = Info_Sup_Recv.objects.get(pk=recv_infofile.pk)
        self.assertEqual(new_info.version, raced_info.version)
        content_file = raced_info.value.read()
        self.assertEqual(content_file, content_img)
        # Set to Null
        form = self.get_user_form(self.reg)
        del form["info-{}-value".format(recv_infofile.pk)]
        form["info-{}-value-clear".format(recv_infofile.pk)] = True
        response4 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response4, reverse("show-registration", args=[self.reg.id]))
        null_info = Info_Sup_Recv.objects.get(pk=recv_infofile.pk)
        self.assertNotEqual(new_info.version, null_info.version)
        self.assertFalse(null_info.value)
        self.assertEqual(null_info.state, Info_Sup_Recv.STATE_MISSING)

    @tag("low", "local")
    def test_InfoFileAccess(self):
        self.infofile.save()
        self.reg.save()
        self.client.force_login(self.user4)
        form = self.get_user_form(self.reg)
        recv_infofile = self.infofile.received.get(registration=self.reg).get_sup_type()
        response = self.client.get(reverse("access-info-sup-file", args=[self.reg.id, recv_infofile.pk]))
        self.assertEqual(response.status_code, 404)
        # Post File
        with open(self.path_img, "rb") as f:
            content_img = f.read()
            img = BytesIO(content_img)
            img.name = "test.png"
        form["info-{}-value".format(recv_infofile.pk)] = img
        response2 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response2, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infofile.pk)
        # access with user
        response3 = self.client.get(new_info.value.url, follow=True)
        self.assertEqual(response3.status_code, 200)
        # access with event admin user
        self.client.force_login(self.useradmin)
        response4 = self.client.get(new_info.value.url, follow=True)
        self.assertEqual(response4.status_code, 200)
        # access with superuser
        self.client.force_login(self.usersuperadmin)
        response5 = self.client.get(new_info.value.url, follow=True)
        self.assertEqual(response5.status_code, 200)
        # access with other user
        self.client.force_login(self.user2)
        response6 = self.client.get(new_info.value.url, follow=True)
        self.assertEqual(response6.status_code, 403)

    @tag("low", "local")
    def test_InfoFileUploadWrongExtension(self):
        self.infofile.save()
        self.reg.save()
        self.client.force_login(self.user4)
        form = self.get_user_form(self.reg)
        recv_infofile = self.infofile.received.get(registration=self.reg).get_sup_type()
        # Post File
        with open(self.path_img, "rb") as f:
            content_img = f.read()
            img = BytesIO(content_img)
            img.name = "test.pdf"
        form["info-{}-value".format(recv_infofile.pk)] = img
        response = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertEqual(response.status_code, 200)
        new_info = Info_Sup_Recv.objects.get(pk=recv_infofile.pk)
        self.assertFalse(new_info.value)

    @tag("fast", "local")
    def test_InfoFileUploadUnallowExtension(self):
        self.infofile.save()
        self.reg.save()
        self.client.force_login(self.user4)
        form = self.get_user_form(self.reg)
        recv_infofile = self.infofile.received.get(registration=self.reg).get_sup_type()
        # Post File
        f = BytesIO(b"#!/bin/sh\nsudo echo 'PWN'\n")
        f.name = "test.sh"
        form["info-{}-value".format(recv_infofile.pk)] = f
        response = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertEqual(response.status_code, 200)
        new_info = Info_Sup_Recv.objects.get(pk=recv_infofile.pk)
        self.assertFalse(new_info.value)

    @tag("fast", "local")
    def test_InfoAutoValidUserForm(self):
        self.infoshorttext.info_validate = True
        self.infoshorttext.save()
        self.reg.save()
        self.client.force_login(self.user4)
        form = self.get_user_form(self.reg)
        recv_infoshorttext = self.infoshorttext.received.get(registration=self.reg).get_sup_type()
        # Post Change
        form["info-{}-value".format(recv_infoshorttext.pk)] = "Personnal Data"
        response = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_ACCEPTED)
        self.assertEqual(new_info.value, "Personnal Data")
        # Ignore other change
        form = self.get_user_form(self.reg)
        form["info-{}-value".format(recv_infoshorttext.pk)] = "Personnal Data 2"
        response2 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response2, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_ACCEPTED)
        self.assertEqual(new_info.value, "Personnal Data")

    @tag("fast", "local")
    def test_InfoEditableUserForm(self):
        self.infoshorttext.info_validate = True
        self.infoshorttext.info_editable = True
        self.infoshorttext.save()
        self.reg.save()
        self.client.force_login(self.user4)
        form = self.get_user_form(self.reg)
        recv_infoshorttext = self.infoshorttext.received.get(registration=self.reg).get_sup_type()
        # Post Change
        form["info-{}-value".format(recv_infoshorttext.pk)] = "Personnal Data"
        response = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_ACCEPTED)
        self.assertEqual(new_info.value, "Personnal Data")
        # Error if blank
        form = self.get_user_form(self.reg)
        form["info-{}-value".format(recv_infoshorttext.pk)] = ""
        response2 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertEqual(response2.status_code, 200)
        new_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_ACCEPTED)
        self.assertEqual(new_info.value, "Personnal Data")
        # Accept Change
        form = self.get_user_form(self.reg)
        form["info-{}-value".format(recv_infoshorttext.pk)] = "Personnal Data 2"
        response3 = self.client.post(reverse("show-registration", args=[self.reg.id]), form)
        self.assertRedirects(response3, reverse("show-registration", args=[self.reg.id]))
        new_info = Info_Sup_Recv.objects.get(pk=recv_infoshorttext.pk)
        self.assertEqual(new_info.state, Info_Sup_Recv.STATE_ACCEPTED)
        self.assertEqual(new_info.value, "Personnal Data 2")


class TestPurchaseTicket(CleanUp, CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    def setUp(self):
        self.tz = pytz.timezone("Europe/Paris")
        self.now = timezone.now().astimezone(self.tz)
        self.event = Event.objects.get(pk=1)
        self.event.tickets.all().delete()
        self.event.start_date = (self.now + datetime.timedelta(days=1)).astimezone(self.tz)
        self.event.end_date = (self.now + datetime.timedelta(days=2)).astimezone(self.tz)
        self.event.save()
        self.restrictedticket = Ticket(
            event=self.event,
            name="RestrictedTicket",
            id=Ticket.next_id(self.event),
            short_notes="short description",
            long_notes="long description",
            price=10.0,
            online_payment=False,
            allowed_external=False,
            limit=0,
            required_school=Ticket.SCHOOL_INP,
            required_ae=False,
            allowed_godson=False,
            admin_ticket=False,
            allow_anonyme=False,
        )
        self.restrictedticket.save()
        self.openticket = Ticket(
            event=self.event,
            name="OpenTicket",
            id=Ticket.next_id(self.event),
            short_notes="short description",
            long_notes="long description",
            price=10.0,
            online_payment=False,
            allowed_external=True,
            limit=0,
            admin_ticket=False,
            allow_anonyme=False,
        )
        self.openticket.save()
        # options alone
        self.optalone1 = Option(
            name="OptAlone1",
            price=5,
            description="Alone is alone",
            group_name=None,
            group=None,
            ticket=self.openticket,
            admin_only=False,
            mandatory=False,
        )
        self.optalone1.save()
        self.optalone2 = Option(
            name="OptAlone2",
            price=10,
            description="Alone is alone",
            group_name=None,
            group=None,
            ticket=self.openticket,
            admin_only=False,
            mandatory=False,
        )
        self.optalone2.save()
        # options group
        self.optgrp1c1 = Option(
            name="Choice1",
            price=15,
            description="Group is grouped",
            group_name="GroupOPT1",
            group=1,
            ticket=self.openticket,
            admin_only=False,
            mandatory=False,
        )
        self.optgrp1c1.save()
        self.optgrp1c2 = Option(
            name="Choice2",
            price=10,
            description="Group is grouped",
            group_name="GroupOPT1",
            group=1,
            ticket=self.openticket,
            admin_only=False,
            mandatory=False,
        )
        self.optgrp1c2.save()
        # info Sup
        self.infoSup1 = Info_Sup(
            ticket=self.openticket,
            name="infosupTest1",
            description="info description",
            after_payement=False,
        )
        self.infoSup1.save()
        self.infoSup2 = Info_Sup(
            ticket=self.openticket,
            name="infosupTest1",
            description="info description",
            after_payement=True,
        )
        self.infoSup2.save()
        # yearp0 correspond à l'année de sortie des 0A à la date de début de l'évènement
        self.yearp0 = self.event.start_date.year + (4 if self.event.start_date.month in [9, 10, 11, 12] else 3)
        # on obtient la promo en effectuant self.yearp0 - user.promo

    ##############################
    # Test purchase by User
    ##############################

    @tag("fast", "local")
    def test_purchaseExte(self):
        user = User.objects.get(email="exte@billetterie.inpt.fr")
        self.client.force_login(user)
        # get event page
        response = self.client.get(reverse("show-event", args=[self.event.id]))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, self.openticket.name)
        self.assertNotContains(response, self.restrictedticket.name)
        # Try get restricted ticket
        response2 = self.client.get(reverse("purchase-ticket", args=[self.event.short_url, self.restrictedticket.id]))
        self.assertRedirects(response2, reverse("show-event", args=[self.event.short_url]))
        # get open ticket
        response3 = self.client.get(reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]))
        self.assertEqual(response3.status_code, 200)
        # pre-purchase
        response4 = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True},
        )
        regs = self.openticket.registrations.filter(
            owner=user, paid=False, payee=None, info_state=Registration.STATE_CANNOT_PAID
        )
        self.assertEqual(len(regs), 1)
        reg = regs.first()
        self.assertEqual(len(reg.options.all()), 0)
        self.assertRedirects(response4, reverse("show-registration", args=[reg.id]))
        # purchase in list of self registration
        response5 = self.client.get(reverse("list-self-registrations"))
        self.assertEqual(response5.status_code, 200)
        self.assertContains(response5, self.event.name)
        self.assertContains(response5, self.openticket.name)
        # show details of registration
        response6 = self.client.get(reverse("list-self-registrations"))
        self.assertEqual(response6.status_code, 200)
        # repost to create a second oneself purchase
        self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True},
        )
        self.assertEqual(len(self.openticket.registrations.filter(owner=user, paid=False, payee=None)), 1)

    @tag("fast", "local")
    def test_purchaseExteGodson(self):
        user = User.objects.get(email="exte@billetterie.inpt.fr")
        self.client.force_login(user)
        # get open ticket
        response = self.client.get(reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]))
        self.assertEqual(response.status_code, 200)
        # pre-purchase
        response2 = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=godson",
            {"conditions_field": True, "payee": "Godson Exte"},
        )
        regs = self.openticket.registrations.filter(owner=user, paid=False, payee="Godson Exte")
        self.assertEqual(len(regs), 1)
        reg = regs.first()
        self.assertRedirects(response2, reverse("show-registration", args=[reg.id]))
        # show details of registration
        response3 = self.client.get(reverse("list-self-registrations"))
        self.assertEqual(response3.status_code, 200)
        self.assertContains(response3, "Godson Exte")

    @tag("fast", "local")
    def test_purchaseExteCancel(self):
        user = User.objects.get(email="exte@billetterie.inpt.fr")
        self.client.force_login(user)
        # pre-purchase
        response = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True},
        )
        regs = self.openticket.registrations.filter(owner=user, paid=False, payee=None)
        self.assertEqual(len(regs), 1)
        reg = regs.first()
        self.assertRedirects(response, reverse("show-registration", args=[reg.id]))
        # show details of registration
        response2 = self.client.get(reverse("list-self-registrations"))
        self.assertEqual(response2.status_code, 200)
        response3 = self.client.post(reverse("cancel-registration", args=[reg.id]))
        self.assertRedirects(response3, reverse("show-event", args=[self.event.short_url]))
        self.assertFalse(self.openticket.registrations.filter(owner=user, paid=False, payee=None).exists())

    @tag("fast", "local")
    def test_purchaseExteWithoutCondition(self):
        user = User.objects.get(email="exte@billetterie.inpt.fr")
        self.client.force_login(user)
        # pre-purchase
        response = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself"
        )
        self.assertTrue(response.status_code, 200)
        regs = self.openticket.registrations.filter(owner=user, paid=False, payee=None)
        self.assertEqual(len(regs), 0)

    @tag("fast", "local")
    def test_purchaseExteFreeTicket(self):
        user = User.objects.get(email="exte@billetterie.inpt.fr")
        self.client.force_login(user)
        self.openticket.infos_sup.all().delete()
        self.openticket.price = 0
        self.openticket.save()
        # pre-purchase
        response = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True},
        )
        regs = self.openticket.registrations.filter(
            owner=user, paid=True, payee=None, info_state=Registration.STATE_CAN_ENTER
        )
        self.assertEqual(len(regs), 1)
        reg = regs.first()
        self.assertRedirects(response, reverse("show-registration", args=[reg.id]))

    @tag("fast", "local")
    def test_purchaseExteBeforeTicketAvailable(self):
        user = User.objects.get(email="exte@billetterie.inpt.fr")
        self.client.force_login(user)
        self.openticket.min_purchase_date = (self.now + datetime.timedelta(hours=1)).astimezone(self.tz)
        self.openticket.save()
        # get
        response = self.client.get(reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]))
        self.assertRedirects(response, reverse("show-event", args=[self.event.short_url]))
        # post
        response2 = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True},
        )
        self.assertRedirects(response2, reverse("show-event", args=[self.event.short_url]))
        self.assertFalse(self.openticket.registrations.filter(owner=user, paid=False, payee=None).exists())

    @tag("fast", "local")
    def test_purchaseExteAfterTicketAvailable(self):
        user = User.objects.get(email="exte@billetterie.inpt.fr")
        self.client.force_login(user)
        self.openticket.max_purchase_date = (self.now - datetime.timedelta(hours=1)).astimezone(self.tz)
        self.openticket.save()
        # get
        response = self.client.get(reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]))
        self.assertRedirects(response, reverse("show-event", args=[self.event.short_url]))
        # post
        response2 = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True},
        )
        self.assertRedirects(response2, reverse("show-event", args=[self.event.short_url]))
        self.assertFalse(self.openticket.registrations.filter(owner=user, paid=False, payee=None).exists())

    @tag("fast", "local")
    def test_purchaseExteAdminTicket(self):
        user = User.objects.get(email="exte@billetterie.inpt.fr")
        self.client.force_login(user)
        self.openticket.admin_ticket = True
        self.openticket.save()
        # get
        response = self.client.get(reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]))
        self.assertRedirects(response, reverse("show-event", args=[self.event.short_url]))
        # post
        response2 = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True},
        )
        self.assertRedirects(response2, reverse("show-event", args=[self.event.short_url]))
        self.assertFalse(self.openticket.registrations.filter(owner=user, paid=False, payee=None).exists())

    @tag("fast", "local")
    def test_purchaseExteOptionAlone(self):
        user = User.objects.get(email="exte@billetterie.inpt.fr")
        self.client.force_login(user)
        # pre-purchase with 1 option
        response = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True, "options": [self.optalone1.pk]},
        )
        regs = self.openticket.registrations.filter(owner=user, paid=False, payee=None)
        self.assertEqual(len(regs), 1)
        reg = regs.first()
        self.assertRedirects(response, reverse("show-registration", args=[reg.id]))
        self.assertEqual(len(reg.options.all()), 1)
        self.assertTrue(reg.options.filter(pk=self.optalone1.pk).exists())
        self.assertFalse(reg.options.filter(pk=self.optalone2.pk).exists())
        reg.delete()
        # pre-purchase with 2 options
        response = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True, "options": [self.optalone1.pk, self.optalone2.pk]},
        )
        regs = self.openticket.registrations.filter(owner=user, paid=False, payee=None)
        self.assertEqual(len(regs), 1)
        reg = regs.first()
        self.assertRedirects(response, reverse("show-registration", args=[reg.id]))
        self.assertEqual(len(reg.options.all()), 2)
        self.assertTrue(reg.options.filter(pk=self.optalone1.pk).exists())
        self.assertTrue(reg.options.filter(pk=self.optalone2.pk).exists())

    @tag("fast", "local")
    def test_purchaseExteInvalidOptionAlone(self):
        user = User.objects.get(email="exte@billetterie.inpt.fr")
        self.client.force_login(user)
        # pre-purchase a grp opt in option alone form
        response = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True, "options": [self.optgrp1c1.pk]},
        )
        self.assertEqual(response.status_code, 200)
        self.assertFalse(self.openticket.registrations.filter(owner=user, paid=False, payee=None).exists())
        # pre-purchase an option alone of another ticket
        self.optalone2.ticket = self.restrictedticket
        self.optalone2.save()
        response2 = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True, "options": [self.optalone2.pk]},
        )
        self.assertEqual(response2.status_code, 200)
        self.assertFalse(self.openticket.registrations.filter(owner=user, paid=False, payee=None).exists())
        # pre-purchase admin option
        self.optalone2.ticket = self.openticket
        self.optalone2.admin_only = True
        self.optalone2.save()
        # with exte account
        response3 = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True, "options": [self.optalone2.pk]},
        )
        self.assertEqual(response3.status_code, 200)
        self.assertFalse(self.openticket.registrations.filter(owner=user, paid=False, payee=None).exists())
        # with admin account
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        response4 = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True, "options": [self.optalone2.pk]},
        )
        self.assertEqual(response4.status_code, 200)
        self.assertFalse(self.openticket.registrations.filter(owner=user, paid=False, payee=None).exists())

    @tag("fast", "local")
    def test_purchaseExteOptionGrp(self):
        user = User.objects.get(email="exte@billetterie.inpt.fr")
        self.client.force_login(user)
        # pre-purchase with first choice
        response = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True, "choice-" + str(self.optgrp1c1.group): self.optgrp1c1.pk},
        )
        regs = self.openticket.registrations.filter(owner=user, paid=False, payee=None)
        self.assertEqual(len(regs), 1)
        reg = regs.first()
        self.assertRedirects(response, reverse("show-registration", args=[reg.id]))
        self.assertEqual(len(reg.options.all()), 1)
        self.assertTrue(reg.options.filter(pk=self.optgrp1c1.pk).exists())
        self.assertFalse(reg.options.filter(pk=self.optgrp1c2.pk).exists())
        reg.delete()
        # pre-purchase with second choice
        response2 = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True, "choice-" + str(self.optgrp1c2.group): self.optgrp1c2.pk},
        )
        regs = self.openticket.registrations.filter(owner=user, paid=False, payee=None)
        self.assertEqual(len(regs), 1)
        reg = regs.first()
        self.assertRedirects(response2, reverse("show-registration", args=[reg.id]))
        self.assertEqual(len(reg.options.all()), 1)
        self.assertFalse(reg.options.filter(pk=self.optgrp1c1.pk).exists())
        self.assertTrue(reg.options.filter(pk=self.optgrp1c2.pk).exists())

    @tag("fast", "local")
    def test_purchaseExteInvalidOptionGrp(self):
        user = User.objects.get(email="exte@billetterie.inpt.fr")
        self.client.force_login(user)
        # pre-purchase with two choices
        response = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True, "choice-" + str(self.optgrp1c1.group): [self.optgrp1c1.pk, self.optgrp1c2.pk]},
        )
        regs = self.openticket.registrations.filter(owner=user, paid=False, payee=None)
        self.assertEqual(len(regs), 1)
        reg = regs.first()
        self.assertRedirects(response, reverse("show-registration", args=[reg.id]))
        # a only choice is associated
        self.assertEqual(len(reg.options.all()), 1)

    @tag("fast", "local")
    def test_purchaseExteInvalidMandatoryOptionGrp(self):
        user = User.objects.get(email="exte@billetterie.inpt.fr")
        self.client.force_login(user)
        self.optgrp1c1.mandatory = True
        self.optgrp1c1.save()
        self.optgrp1c2.mandatory = True
        self.optgrp1c2.save()
        # pre-purchase with any choice
        response = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True},
        )
        self.assertEqual(response.status_code, 200)
        self.assertFalse(self.openticket.registrations.filter(owner=user).exists())

    @tag("low", "local")
    def test_purchaseExteLimitedTicket(self):
        user = User.objects.get(email="exte@billetterie.inpt.fr")
        self.client.force_login(user)
        self.openticket.limit = 10
        self.openticket.save()
        # purchase 10
        for i in range(10):
            response = self.client.post(
                reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=godson",
                {"conditions_field": True, "payee": "Godson Exte " + str(i)},
            )
            regs = self.openticket.registrations.filter(owner=user, paid=False, payee="Godson Exte " + str(i))
            self.assertEqual(len(regs), 1)
            reg = regs.first()
            self.assertRedirects(response, reverse("show-registration", args=[reg.id]))
            # mark as paid
            reg.paid = True
            reg.save()

        # get pre-purchase
        response2 = self.client.get(reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]))
        self.assertRedirects(response2, reverse("show-event", args=[self.event.short_url]))
        # post pre-purchase
        response3 = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True},
        )
        self.assertRedirects(response3, reverse("show-event", args=[self.event.short_url]))
        self.assertFalse(self.openticket.registrations.filter(owner=user, paid=False, payee=None).exists())

    @tag("low", "local")
    def test_purchaseLimitedEvent(self):
        user = User.objects.get(email="exte@billetterie.inpt.fr")
        user2 = User.objects.get(login_inp_net="testone")
        self.openticket.limit = 20
        self.openticket.save()
        self.restrictedticket.limit = 20
        self.restrictedticket.allowed_godson = True
        self.restrictedticket.save()
        self.event.limit_godson = 20
        self.event.limit_ticket = 20
        self.event.save()
        # purchase 10 Restrict
        self.client.force_login(user2)
        for i in range(10):
            response = self.client.post(
                reverse("purchase-ticket", args=[self.event.short_url, self.restrictedticket.id]) + "?payee=godson",
                {"conditions_field": True, "payee": "Godson Rest " + str(i)},
            )
            regs = self.restrictedticket.registrations.filter(owner=user2, paid=False, payee="Godson Rest " + str(i))
            self.assertEqual(len(regs), 1)
            reg = regs.first()
            self.assertRedirects(response, reverse("show-registration", args=[reg.id]))
            # mark as paid
            reg.paid = True
            reg.save()

        # purchase 10 Exte
        self.client.force_login(user)
        for i in range(10):
            response = self.client.post(
                reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=godson",
                {"conditions_field": True, "payee": "Godson Exte " + str(i)},
            )
            regs = self.openticket.registrations.filter(owner=user, paid=False, payee="Godson Exte " + str(i))
            self.assertEqual(len(regs), 1)
            reg = regs.first()
            self.assertRedirects(response, reverse("show-registration", args=[reg.id]))
            # mark as paid
            reg.paid = True
            reg.save()

        # get pre-purchase Exte
        response2 = self.client.get(reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]))
        self.assertRedirects(response2, reverse("show-event", args=[self.event.short_url]))
        # post pre-purchase Exte
        response3 = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.openticket.id]) + "?payee=oneself",
            {"conditions_field": True},
        )
        self.assertRedirects(response3, reverse("show-event", args=[self.event.short_url]))
        self.assertFalse(self.openticket.registrations.filter(owner=user, paid=False, payee=None).exists())

        self.client.force_login(user2)
        # get pre-purchase Exte
        response4 = self.client.get(reverse("purchase-ticket", args=[self.event.short_url, self.restrictedticket.id]))
        self.assertRedirects(response4, reverse("show-event", args=[self.event.short_url]))
        self.assertFalse(self.restrictedticket.registrations.filter(owner=user2, paid=False, payee=None).exists())

    @tag("fast", "local")
    def test_purchaseRestrictGodsonNotAvailable(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        response = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.restrictedticket.id]) + "?payee=godson",
            {"conditions_field": True, "payee": "Godson Rest"},
        )
        self.assertRedirects(response, reverse("show-event", args=[self.event.short_url]))
        self.assertFalse(
            self.restrictedticket.registrations.filter(owner=user, paid=False, payee="Godson Rest").exists()
        )

    @tag("low", "local")
    def test_purchaseRestrictGodsonLimit(self):
        user = User.objects.get(login_inp_net="testone")
        self.restrictedticket.allowed_godson = True
        self.restrictedticket.save()
        self.event.limit_godson = 10
        self.event.save()
        # purchase 10 Restrict
        self.client.force_login(user)
        for i in range(10):
            response = self.client.post(
                reverse("purchase-ticket", args=[self.event.short_url, self.restrictedticket.id]) + "?payee=godson",
                {"conditions_field": True, "payee": "Godson Rest " + str(i)},
            )
            regs = self.restrictedticket.registrations.filter(owner=user, paid=False, payee="Godson Rest " + str(i))
            self.assertEqual(len(regs), 1)
            reg = regs.first()
            self.assertRedirects(response, reverse("show-registration", args=[reg.id]))
            # mark as paid
            reg.paid = True
            reg.save()
        # get another godson
        response2 = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.restrictedticket.id]) + "?payee=godson",
            {"conditions_field": True, "payee": "Godson Rest"},
        )
        self.assertRedirects(response2, reverse("show-event", args=[self.event.short_url]))
        self.assertFalse(
            self.restrictedticket.registrations.filter(owner=user, paid=False, payee="Godson Rest").exists()
        )

    @tag("fast", "local")
    def test_purchaseRestrictAdminTicket(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        self.restrictedticket.admin_ticket = True
        self.restrictedticket.save()
        # get
        response = self.client.get(reverse("purchase-ticket", args=[self.event.short_url, self.restrictedticket.id]))
        self.assertRedirects(response, reverse("show-event", args=[self.event.short_url]))
        # post
        response2 = self.client.post(
            reverse("purchase-ticket", args=[self.event.short_url, self.restrictedticket.id]) + "?payee=oneself",
            {"conditions_field": True},
        )
        self.assertRedirects(response2, reverse("show-event", args=[self.event.short_url]))
        self.assertFalse(self.restrictedticket.registrations.filter(owner=user, paid=False, payee=None).exists())

    @tag("low", "local")
    def test_purchaseRestrictAdherentAE(self):
        self.restrictedticket.required_ae = True
        self.restrictedticket.save()
        for user in User.objects.filter(is_anonyme=False):
            self.client.force_login(user)
            # get
            response = self.client.get(
                reverse("purchase-ticket", args=[self.event.short_url, self.restrictedticket.id])
            )
            # post
            response2 = self.client.post(
                reverse("purchase-ticket", args=[self.event.short_url, self.restrictedticket.id]) + "?payee=oneself",
                {"conditions_field": True},
            )
            if user.is_adherent_ae:
                self.assertEqual(response.status_code, 200)
                regs = self.restrictedticket.registrations.filter(owner=user, paid=False, payee=None)
                self.assertEqual(len(regs), 1)
                reg = regs.first()
                self.assertRedirects(response2, reverse("show-registration", args=[reg.id]))
                reg.delete()
            else:
                self.assertRedirects(response, reverse("show-event", args=[self.event.short_url]))
                self.assertRedirects(response2, reverse("show-event", args=[self.event.short_url]))
                self.assertFalse(
                    self.restrictedticket.registrations.filter(owner=user, paid=False, payee=None).exists()
                )

    @tag("low", "local")
    def test_purchaseRestrictEcole(self):
        for restrict, ecole in [
            (Ticket.SCHOOL_INP, ["n7", "ensat", "a7"]),
            (Ticket.SCHOOL_N7, ["n7"]),
            (Ticket.SCHOOL_ENSAT, ["ensat"]),
            (Ticket.SCHOOL_A7, ["a7"]),
        ]:
            self.restrictedticket.required_school = restrict
            self.restrictedticket.save()
            for user in User.objects.filter(is_anonyme=False):
                self.client.force_login(user)
                # get
                response = self.client.get(
                    reverse("purchase-ticket", args=[self.event.short_url, self.restrictedticket.id])
                )
                # post
                response2 = self.client.post(
                    reverse("purchase-ticket", args=[self.event.short_url, self.restrictedticket.id])
                    + "?payee=oneself",
                    {"conditions_field": True},
                )
                if user.ecole in ecole:
                    self.assertEqual(response.status_code, 200)
                    regs = self.restrictedticket.registrations.filter(owner=user, paid=False, payee=None)
                    self.assertEqual(len(regs), 1)
                    reg = regs.first()
                    self.assertRedirects(response2, reverse("show-registration", args=[reg.id]))
                    reg.delete()
                else:
                    self.assertRedirects(response, reverse("show-event", args=[self.event.short_url]))
                    self.assertRedirects(response2, reverse("show-event", args=[self.event.short_url]))
                    self.assertFalse(
                        self.restrictedticket.registrations.filter(owner=user, paid=False, payee=None).exists()
                    )

    @tag("low", "local")
    def test_purchaseRestrictPromo(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        for restrict, checkpromo in [
            ([], lambda x: True),
            ([Ticket.YEAR_1A], lambda x: 1 == x),
            ([Ticket.YEAR_2A], lambda x: 2 == x),
            ([Ticket.YEAR_3A], lambda x: 3 == x),
            ([Ticket.YEAR_4A], lambda x: 4 == x),
            ([Ticket.YEAR_5APLUS], lambda x: 5 <= x),
            ([Ticket.YEAR_4A, Ticket.YEAR_5APLUS], lambda x: 4 <= x),
            ([Ticket.YEAR_1A, Ticket.YEAR_2A, Ticket.YEAR_3A], lambda x: x in [1, 2, 3]),
        ]:

            self.restrictedticket.year_choice = map(str, restrict) if restrict else None
            self.restrictedticket.save()
            for promo in range(8):
                user.promo = self.yearp0 - promo
                user.save()
                # get
                response = self.client.get(
                    reverse("purchase-ticket", args=[self.event.short_url, self.restrictedticket.id])
                )
                # post
                response2 = self.client.post(
                    reverse("purchase-ticket", args=[self.event.short_url, self.restrictedticket.id])
                    + "?payee=oneself",
                    {"conditions_field": True},
                )
                if checkpromo(promo):
                    self.assertEqual(response.status_code, 200)
                    regs = self.restrictedticket.registrations.filter(owner=user, paid=False, payee=None)
                    self.assertEqual(len(regs), 1)
                    reg = regs.first()
                    self.assertRedirects(response2, reverse("show-registration", args=[reg.id]))
                    reg.delete()
                else:
                    self.assertRedirects(response, reverse("show-event", args=[self.event.short_url]))
                    self.assertRedirects(response2, reverse("show-event", args=[self.event.short_url]))
                    self.assertFalse(
                        self.restrictedticket.registrations.filter(owner=user, paid=False, payee=None).exists()
                    )

    ##############################
    # Test purchase by Admin
    ##############################

    @tag("fast", "local")
    def test_adminPurchaseExteTicket(self):
        user = User.objects.get(login_inp_net="testone")
        user2 = User.objects.get(email="exte@billetterie.inpt.fr")
        self.optalone1.admin_only = True
        self.optalone1.save()
        self.client.force_login(user)
        response = self.client.get(reverse("choice-registration", args=[self.event.short_url]))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, self.restrictedticket.name)
        self.assertContains(response, self.openticket.name)
        # purchase a billet to an exte with option
        response2 = self.client.get(reverse("new-registration", args=[self.event.short_url, self.openticket.id]))
        self.assertEqual(response2.status_code, 200)
        self.assertContains(response2, self.optalone1.name)  # l'option admin est visible
        response3 = self.client.post(
            reverse("new-registration", args=[self.event.short_url, self.openticket.id]),
            {
                "choice-" + str(self.optgrp1c1.group): self.optgrp1c1.pk,
                "options": [self.optalone1.pk, self.optalone2.pk],
                "email": user2.email,
                "login": "",
                "payee": "",
                "conditions_field": True,
            },
        )
        regs = self.openticket.registrations.filter(
            owner=user2, paid=False, payee=None, info_state=Registration.STATE_CANNOT_PAID
        )
        self.assertEqual(len(regs), 1)
        reg = regs.first()
        self.assertRedirects(response3, reverse("registration-edit", args=[self.event.short_url, reg.id]))
        self.assertEqual(len(reg.options.all()), 3)
        self.assertTrue(reg.options.filter(pk=self.optalone1.pk).exists())
        self.assertTrue(reg.options.filter(pk=self.optalone2.pk).exists())
        self.assertTrue(reg.options.filter(pk=self.optgrp1c1.pk).exists())
        for info in reg.info_sup_received.all():
            self.assertEqual(info.state, Info_Sup_Recv.STATE_MISSING)
        # Do a second purchase with the same user
        response4 = self.client.post(
            reverse("new-registration", args=[self.event.short_url, self.openticket.id]),
            {
                "choice-" + str(self.optgrp1c1.group): self.optgrp1c1.pk,
                "options": [self.optalone1.pk, self.optalone2.pk],
                "email": user2.email,
                "login": "",
                "payee": "",
                "conditions_field": True,
            },
        )
        self.assertEqual(response4.status_code, 200)
        regs = self.openticket.registrations.filter(
            owner=user2, paid=False, payee=None, info_state=Registration.STATE_CANNOT_PAID
        )
        self.assertEqual(len(regs), 1)

    @tag("fast", "local")
    def test_adminPurchaseExteFreeTicket(self):
        user = User.objects.get(login_inp_net="testone")
        user2 = User.objects.get(email="exte@billetterie.inpt.fr")
        self.openticket.infos_sup.all().delete()
        self.openticket.price = 0
        self.openticket.save()
        self.client.force_login(user)
        # purchase a billet to an exte with option and
        response = self.client.post(
            reverse("new-registration", args=[self.event.short_url, self.openticket.id]),
            {"email": user2.email, "login": "", "payee": "", "conditions_field": True},
        )
        regs = self.openticket.registrations.filter(
            owner=user2, paid=True, payee=None, info_state=Registration.STATE_CAN_ENTER
        )
        self.assertEqual(len(regs), 1)
        reg = regs.first()
        self.assertRedirects(response, reverse("registration-edit", args=[self.event.short_url, reg.id]))
        self.assertEqual(len(reg.options.all()), 0)
        self.assertEqual(len(reg.info_sup_received.all()), 0)

    @tag("fast", "local")
    def test_adminPurchaseINPUserTicket(self):
        user = User.objects.get(login_inp_net="testone")
        user2 = User.objects.get(login_inp_net="testwo")
        self.client.force_login(user)
        # purchase a billet to an exte with option
        response = self.client.post(
            reverse("new-registration", args=[self.event.short_url, self.openticket.id]),
            {
                "choice-" + str(self.optgrp1c1.group): self.optgrp1c2.pk,
                "options": [self.optalone1.pk, self.optalone2.pk],
                "email": "",
                "login": user2.login_inp_net,
                "payee": "",
                "conditions_field": True,
            },
        )
        regs = self.openticket.registrations.filter(
            owner=user2, paid=False, payee=None, info_state=Registration.STATE_CANNOT_PAID
        )
        self.assertEqual(len(regs), 1)
        reg = regs.first()
        self.assertRedirects(response, reverse("registration-edit", args=[self.event.short_url, reg.id]))
        self.assertEqual(len(reg.options.all()), 3)
        self.assertTrue(reg.options.filter(pk=self.optalone1.pk).exists())
        self.assertTrue(reg.options.filter(pk=self.optalone2.pk).exists())
        self.assertTrue(reg.options.filter(pk=self.optgrp1c2.pk).exists())
        for info in reg.info_sup_received.all():
            self.assertEqual(info.state, Info_Sup_Recv.STATE_MISSING)

    @tag("fast", "local")
    def test_adminPurchaseExteBeforeTicketAvailable(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        self.openticket.min_purchase_date = (self.now + datetime.timedelta(hours=1)).astimezone(self.tz)
        self.openticket.save()
        response = self.client.get(reverse("new-registration", args=[self.event.short_url, self.openticket.id]))
        self.assertRedirects(response, reverse("choice-registration", args=[self.event.short_url]))

    @tag("fast", "local")
    def test_adminPurchaseExteAfterTicketAvailable(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        self.openticket.max_purchase_date = (self.now - datetime.timedelta(hours=1)).astimezone(self.tz)
        self.openticket.save()
        response = self.client.get(reverse("new-registration", args=[self.event.short_url, self.openticket.id]))
        self.assertRedirects(response, reverse("choice-registration", args=[self.event.short_url]))

    @tag("fast", "local")
    def test_adminPurchaseExteTicketNotAvailable(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        self.openticket.infos_sup.all().delete()
        self.openticket.limit = 10
        self.openticket.save()
        for i in range(10):
            Registration(ticket=self.openticket, owner=user, payee="Test" + str(i), paid=True).save()
        response = self.client.get(reverse("new-registration", args=[self.event.short_url, self.openticket.id]))
        self.assertRedirects(response, reverse("choice-registration", args=[self.event.short_url]))

    @tag("fast", "local")
    def test_adminPurchaseExteTicketEventLimit(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        self.openticket.infos_sup.all().delete()
        self.openticket.save()
        self.event.limit_ticket = 10
        self.event.save()
        for i in range(10):
            Registration(ticket=self.openticket, owner=user, payee="Test" + str(i), paid=True).save()
        response = self.client.get(reverse("new-registration", args=[self.event.short_url, self.openticket.id]))
        self.assertRedirects(response, reverse("choice-registration", args=[self.event.short_url]))

    @tag("fast", "local")
    def test_adminPurchaseRestrictedGodsonLimit(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        self.restrictedticket.allowed_godson = True
        self.restrictedticket.save()
        self.event.limit_godson = 10
        self.event.save()
        for i in range(10):
            Registration(ticket=self.restrictedticket, owner=user, payee="Test" + str(i)).save()
        response = self.client.post(
            reverse("new-registration", args=[self.event.short_url, self.restrictedticket.id]),
            {"email": "", "login": user.login_inp_net, "payee": "Godson", "conditions_field": True},
        )
        self.assertFalse(self.restrictedticket.registrations.filter(owner=user, payee="Godson").exists())
        self.assertEqual(response.status_code, 200)

    @tag("fast", "local")
    def test_adminPurchaseInvalidForm(self):
        user = User.objects.get(login_inp_net="testone")
        user2 = User.objects.get(login_inp_net="testwo")
        user3 = User.objects.get(email="exte@billetterie.inpt.fr")
        self.openticket.allow_anonyme = True
        self.openticket.save()
        self.client.force_login(user)
        # many user selected
        response = self.client.post(
            reverse("new-registration", args=[self.event.short_url, self.openticket.id]),
            {"email": user3.email, "login": user2.login_inp_net, "payee": "", "conditions_field": True},
        )
        self.assertFalse(self.openticket.registrations.filter(owner=user2).exists())
        self.assertFalse(self.openticket.registrations.filter(owner=user3).exists())
        self.assertEqual(response.status_code, 200)
        # no user selected
        response2 = self.client.post(
            reverse("new-registration", args=[self.event.short_url, self.openticket.id]), {"conditions_field": True}
        )
        self.assertEqual(response2.status_code, 200)
        # user not in BDD
        self.assertFalse(User.objects.filter(email="no-existed-user@billetterie.inpt.fr").exists())
        response3 = self.client.post(
            reverse("new-registration", args=[self.event.short_url, self.openticket.id]),
            {"email": "no-existed-user@billetterie.inpt.fr", "login": "", "payee": "", "conditions_field": True},
        )
        self.assertEqual(response3.status_code, 200)
        # condition not accepted
        response4 = self.client.post(
            reverse("new-registration", args=[self.event.short_url, self.openticket.id]),
            {"email": "", "login": user2.login_inp_net, "payee": "", "conditions_field": False},
        )
        self.assertFalse(self.openticket.registrations.filter(owner=user2).exists())
        self.assertEqual(response4.status_code, 200)
        # anonyme without name of buyer
        response5 = self.client.post(
            reverse("new-registration", args=[self.event.short_url, self.openticket.id]),
            {"email": "", "login": "", "anonyme": True, "payee": "", "conditions_field": True},
        )
        self.assertFalse(self.openticket.registrations.filter(owner=user.get_anonyme_user()).exists())
        self.assertEqual(response5.status_code, 200)
        # godson with restricted and not godson available
        response5 = self.client.post(
            reverse("new-registration", args=[self.event.short_url, self.restrictedticket.id]),
            {"email": "", "login": user2.login_inp_net, "payee": "Godson", "conditions_field": True},
        )
        self.assertFalse(self.restrictedticket.registrations.filter(owner=user2).exists())
        self.assertEqual(response5.status_code, 200)
        # mandatory option
        self.optgrp1c1.mandatory = True
        self.optgrp1c1.save()
        self.optgrp1c2.mandatory = True
        self.optgrp1c2.save()
        response6 = self.client.post(
            reverse("new-registration", args=[self.event.short_url, self.openticket.id]),
            {"email": "", "login": user2.login_inp_net, "payee": "", "conditions_field": True},
        )
        self.assertEqual(response6.status_code, 200)
        self.assertFalse(self.openticket.registrations.filter(owner=user2).exists())

    @tag("fast", "local")
    def test_adminPurchaseExteAnonymeTicket(self):
        user = User.objects.get(login_inp_net="testone")
        self.openticket.allow_anonyme = True
        self.openticket.save()
        self.client.force_login(user)
        response = self.client.post(
            reverse("new-registration", args=[self.event.short_url, self.openticket.id]),
            {"email": "", "login": "", "anonyme": True, "payee": "Anonyme Test", "conditions_field": True},
        )
        regs = self.openticket.registrations.filter(owner=user.get_anonyme_user(), payee="Anonyme Test", paid=False)
        self.assertEqual(len(regs), 1)
        reg = regs.first()
        self.assertRedirects(response, reverse("registration-edit", args=[self.event.short_url, reg.id]))

    ##############################
    # Test List (pre)reservation
    ##############################

    @tag("fast", "local")
    def test_listPreReservation(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.restrictedticket, owner=user, payee=None, paid=False)
        reg.save()
        response = self.client.get(reverse("list-unpaid-registrations", args=[self.event.short_url]))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "<td>{}</td>".format(reg.pk))

    @tag("fast", "local")
    def test_listReservation(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.restrictedticket, owner=user, payee=None, paid=True)
        reg.save()
        response = self.client.get(reverse("list-paid-registrations", args=[self.event.short_url]))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "<td>{}</td>".format(reg.pk))

    @tag("fast", "local")
    def test_printlistReservation(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.openticket, owner=user, payee=None, paid=True)
        reg.save()
        reg.options.add(self.optalone1)
        reg.options.add(self.optgrp1c2)
        info = Info_Sup_Recv.objects.get(reference=self.infoSup1, registration=reg)
        info.state = Info_Sup_Recv.STATE_ACCEPTED
        info.save()
        info2 = Info_Sup_Recv.objects.get(reference=self.infoSup2, registration=reg)
        info2.state = Info_Sup_Recv.STATE_ACCEPTED
        info2.save()
        reg.save()
        maj_state_registration(reg)
        response = self.client.get(reverse("download-registrations", args=[self.event.short_url]))
        self.assertEqual(response.status_code, 200)
        path = os.path.join(settings.BASE_DIR, "guichet", "tests", "result_pdf", "list_registration.pdf")
        with open(path, "wb") as f:
            f.write(response.content)

    ##############################
    # Validate Info_Sup (Old Version adapted)
    ##############################

    @tag("fast", "local")
    def test_validate1Infos_sup(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.openticket, owner=user, payee=None, paid=True)
        reg.save()
        response = self.client.get(reverse("registration-edit", args=[self.event.short_url, reg.pk]))
        self.assertEqual(response.status_code, 200)
        info_sup1 = self.infoSup1.received.get(registration=reg)
        info_sup2 = self.infoSup2.received.get(registration=reg)
        form = {
            "info-{}-uuid".format(info_sup1.pk): info_sup1.version,
            "info-{}-state".format(info_sup1.pk): str(Info_Sup_Recv.STATE_ACCEPTED),
            "info-{}-uuid".format(info_sup2.pk): info_sup2.version,
            "info-{}-state".format(info_sup2.pk): info_sup2.state,
        }
        response2 = self.client.post(reverse("registration-edit", args=[self.event.short_url, reg.pk]), form)
        self.assertRedirects(response2, reverse("registration-edit", args=[self.event.short_url, reg.pk]))
        new_reg = Registration.objects.get(pk=reg.pk)
        infos_sup = new_reg.info_sup_received.filter(state=Info_Sup_Recv.STATE_ACCEPTED)
        self.assertEqual(len(infos_sup), 1)
        self.assertTrue(infos_sup.filter(reference__pk=self.infoSup1.pk).exists())
        self.assertFalse(infos_sup.filter(reference__pk=self.infoSup2.pk).exists())

    @tag("fast", "local")
    def test_validate2Infos_sup(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.openticket, owner=user, payee=None, paid=True)
        reg.save()
        response = self.client.get(reverse("registration-edit", args=[self.event.short_url, reg.pk]))
        self.assertEqual(response.status_code, 200)
        info_sup1 = self.infoSup1.received.get(registration=reg)
        info_sup2 = self.infoSup2.received.get(registration=reg)
        form = {
            "info-{}-uuid".format(info_sup1.pk): info_sup1.version,
            "info-{}-state".format(info_sup1.pk): str(Info_Sup_Recv.STATE_ACCEPTED),
            "info-{}-uuid".format(info_sup2.pk): info_sup2.version,
            "info-{}-state".format(info_sup2.pk): str(Info_Sup_Recv.STATE_ACCEPTED),
        }
        response2 = self.client.post(reverse("registration-edit", args=[self.event.short_url, reg.pk]), form)
        self.assertRedirects(response2, reverse("registration-edit", args=[self.event.short_url, reg.pk]))
        new_reg = Registration.objects.get(pk=reg.pk)
        infos_sup = new_reg.info_sup_received.filter(state=Info_Sup_Recv.STATE_ACCEPTED)
        self.assertEqual(len(infos_sup), 2)
        self.assertTrue(infos_sup.filter(reference__pk=self.infoSup1.pk).exists())
        self.assertTrue(infos_sup.filter(reference__pk=self.infoSup2.pk).exists())

    @tag("fast", "local")
    def test_validate1Infos_supFreeTicket(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        self.openticket.price = 0
        self.openticket.save()
        reg = Registration(ticket=self.openticket, owner=user, payee=None, paid=False)
        reg.save()
        self.assertEqual(reg.info_state, Registration.STATE_CANNOT_PAID)
        info_sup1 = self.infoSup1.received.get(registration=reg)
        info_sup2 = self.infoSup2.received.get(registration=reg)
        form = {
            "info-{}-uuid".format(info_sup1.pk): info_sup1.version,
            "info-{}-state".format(info_sup1.pk): str(Info_Sup_Recv.STATE_ACCEPTED),
            "info-{}-uuid".format(info_sup2.pk): info_sup2.version,
            "info-{}-state".format(info_sup2.pk): str(Info_Sup_Recv.STATE_ACCEPTED),
        }
        response = self.client.post(reverse("registration-edit", args=[self.event.short_url, reg.pk]), form)
        self.assertRedirects(response, reverse("registration-edit", args=[self.event.short_url, reg.pk]))
        new_reg = Registration.objects.get(pk=reg.pk)
        self.assertTrue(new_reg.paid)
        infos_sup = new_reg.info_sup_received.filter(state=Info_Sup_Recv.STATE_ACCEPTED)
        self.assertEqual(len(infos_sup), 2)
        self.assertTrue(infos_sup.filter(reference__pk=self.infoSup1.pk).exists())
        self.assertTrue(infos_sup.filter(reference__pk=self.infoSup2.pk).exists())

    ##############################
    # Validate registration
    ##############################

    @tag("fast", "local")
    def test_validateRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.restrictedticket, owner=user, payee=None)
        reg.save()
        response = self.client.post(
            reverse("validate-registration-method", args=[self.event.short_url, reg.pk, Registration.PAYMENT_CASH])
        )
        self.assertRedirects(response, reverse("list-paid-registrations", args=[self.event.short_url]))
        self.assertTrue(
            Registration.objects.filter(pk=reg.pk, paid=True, payment_method=Registration.PAYMENT_CASH).exists()
        )
        # test repost for coverage
        response2 = self.client.post(
            reverse("validate-registration-method", args=[self.event.short_url, reg.pk, Registration.PAYMENT_CHECK])
        )
        self.assertRedirects(response2, reverse("list-paid-registrations", args=[self.event.short_url]))
        self.assertFalse(
            Registration.objects.filter(pk=reg.pk, paid=True, payment_method=Registration.PAYMENT_CHECK).exists()
        )

    @tag("fast", "local")
    def test_validateCannotPaidRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.openticket, owner=user, payee=None)
        reg.save()
        self.assertEqual(reg.info_state, Registration.STATE_CANNOT_PAID)
        response = self.client.post(
            reverse("validate-registration-method", args=[self.event.short_url, reg.pk, Registration.PAYMENT_CASH])
        )
        self.assertRedirects(response, reverse("list-unpaid-registrations", args=[self.event.short_url]))
        self.assertTrue(Registration.objects.filter(pk=reg.pk, paid=False).exists())

    @tag("fast", "local")
    def test_validateForcePaidRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        self.event.limit_ticket = 10
        self.event.save()
        for i in range(10):
            Registration(ticket=self.openticket, owner=user, payee="Test{}".format(i), paid=True).save()
        reg = Registration(ticket=self.restrictedticket, owner=user, payee=None)
        reg.save()
        response = self.client.post(reverse("validate-registration", args=[self.event.short_url, reg.pk]))
        self.assertRedirects(response, reverse("list-unpaid-registrations", args=[self.event.short_url]))
        self.assertTrue(Registration.objects.filter(pk=reg.pk, paid=False).exists())
        # force
        response2 = self.client.post(reverse("validate-registration", args=[self.event.short_url, reg.pk]) + "?force")
        self.assertRedirects(response2, reverse("list-paid-registrations", args=[self.event.short_url]))
        self.assertTrue(
            Registration.objects.filter(pk=reg.pk, paid=True, payment_method=Registration.PAYMENT_UNKNOW).exists()
        )

    @tag("fast", "local")
    def test_validateRegistrationWithoutInfoSupAfterPayment(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.openticket, owner=user, payee=None)
        reg.save()
        self.assertEqual(reg.info_state, Registration.STATE_CANNOT_PAID)
        info = Info_Sup_Recv.objects.get(reference=self.infoSup1, registration=reg)
        info.state = Info_Sup_Recv.STATE_ACCEPTED
        info.save()
        reg.save()
        self.assertEqual(reg.info_state, Registration.STATE_CAN_PAID)
        response = self.client.post(
            reverse("validate-registration-method", args=[self.event.short_url, reg.pk, Registration.PAYMENT_CASH])
        )
        self.assertRedirects(response, reverse("list-paid-registrations", args=[self.event.short_url]))
        self.assertTrue(
            Registration.objects.filter(
                pk=reg.pk, paid=True, payment_method=Registration.PAYMENT_CASH, info_state=Registration.STATE_CAN_PAID
            ).exists()
        )
        reg = Registration.objects.get(pk=reg.pk)
        test_info = Info_Sup_Recv.objects.get(reference=self.infoSup2, registration=reg)
        self.assertEqual(test_info.state, Info_Sup_Recv.STATE_MISSING)
        test_info.state = Info_Sup_Recv.STATE_REJECTED
        test_info.save()
        reg.save()
        self.assertEqual(reg.info_state, Registration.STATE_CAN_PAID)
        test_info.state = Info_Sup_Recv.STATE_PENDING
        test_info.save()
        reg.save()
        self.assertEqual(reg.info_state, Registration.STATE_CAN_PAID)
        test_info.state = Info_Sup_Recv.STATE_ACCEPTED
        test_info.save()
        reg.save()
        self.assertEqual(reg.info_state, Registration.STATE_CAN_ENTER)

    ##############################
    # Test Paypal Return
    ##############################

    @tag("fast", "local")
    def test_paypalreturnPaid(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.openticket, owner=user, paid=True)
        reg.save()
        response = self.client.post(reverse("paypal-return-registration", args=[reg.pk]))
        self.assertRedirects(response, reverse("show-registration", args=[reg.pk]))
        response2 = self.client.get(reverse("paypal-return-registration", args=[reg.pk]))
        self.assertRedirects(response2, reverse("show-registration", args=[reg.pk]))

    @tag("fast", "local")
    def test_paypalreturnNotPaid(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.openticket, owner=user, paid=False)
        reg.save()
        response = self.client.post(reverse("paypal-return-registration", args=[reg.pk]))
        self.assertRedirects(response, reverse("show-registration", args=[reg.pk]))
        response2 = self.client.get(reverse("paypal-return-registration", args=[reg.pk]))
        self.assertRedirects(response2, reverse("show-registration", args=[reg.pk]))

    @tag("fast", "local")
    def test_paypalreturnCancel(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.openticket, owner=user, paid=False)
        reg.save()
        response = self.client.post(reverse("paypal-cancel-registration", args=[reg.pk]))
        self.assertRedirects(response, reverse("show-registration", args=[reg.pk]))
        response2 = self.client.get(reverse("paypal-cancel-registration", args=[reg.pk]))
        self.assertRedirects(response2, reverse("show-registration", args=[reg.pk]))

    ##############################
    # Test Opposition
    ##############################

    @tag("fast", "local")
    def test_oppositionAdmin(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.openticket, owner=user, paid=True)
        reg.save()
        response = self.client.post(reverse("oppose-registration", args=[self.event.short_url, reg.pk, "norefund"]))
        self.assertRedirects(response, reverse("list-paid-registrations", args=[self.event.short_url]))
        reg = Registration.objects.get(pk=reg.pk)
        self.assertTrue(reg.opposed)
        self.assertFalse(reg.refund)
        self.assertEqual(reg.reason_opposed, Registration.OPPOSED_BY_ADMIN)
        response2 = self.client.post(
            reverse("oppose-registration", args=[self.event.short_url, reg.pk, "remove"]) + "?details"
        )
        self.assertRedirects(response2, reverse("registration-edit", args=[self.event.short_url, reg.id]))
        reg = Registration.objects.get(pk=reg.pk)
        self.assertFalse(reg.opposed)
        self.assertFalse(reg.refund)

    @tag("fast", "local")
    def test_oppositionRefund(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.openticket, owner=user, paid=True)
        reg.save()
        response = self.client.post(reverse("oppose-registration", args=[self.event.short_url, reg.pk, "refund"]))
        self.assertRedirects(response, reverse("list-paid-registrations", args=[self.event.short_url]))
        reg = Registration.objects.get(pk=reg.pk)
        self.assertTrue(reg.opposed)
        self.assertTrue(reg.refund)
        self.assertEqual(reg.reason_opposed, Registration.REFUND_BY_ADMIN)
        response2 = self.client.post(
            reverse("oppose-registration", args=[self.event.short_url, reg.pk, "remove"]) + "?details"
        )
        self.assertRedirects(response2, reverse("registration-edit", args=[self.event.short_url, reg.id]))
        reg = Registration.objects.get(pk=reg.pk)
        self.assertFalse(reg.opposed)
        self.assertFalse(reg.refund)

    @tag("fast", "local")
    def test_oppositionPaypal(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(
            ticket=self.openticket, owner=user, paid=True, opposed=True, reason_opposed=Registration.OPPOSED_BY_PAYPAL
        )
        reg.save()
        response = self.client.post(reverse("oppose-registration", args=[self.event.short_url, reg.pk, "norefund"]))
        self.assertRedirects(response, reverse("list-paid-registrations", args=[self.event.short_url]))
        reg = Registration.objects.get(pk=reg.pk)
        self.assertTrue(reg.opposed)
        self.assertFalse(reg.refund)
        self.assertEqual(reg.reason_opposed, Registration.OPPOSED_BY_PAYPAL)
        response2 = self.client.post(
            reverse("oppose-registration", args=[self.event.short_url, reg.pk, "remove"]) + "?force&details"
        )
        self.assertRedirects(response2, reverse("registration-edit", args=[self.event.short_url, reg.id]))
        reg = Registration.objects.get(pk=reg.pk)
        self.assertFalse(reg.opposed)
        self.assertFalse(reg.refund)

    ##############################
    # Test Check
    ##############################

    @tag("fast", "local")
    def test_checkByLoginINPnet(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.restrictedticket, owner=user, paid=True)
        reg.save()
        response = self.client.post(
            reverse("check-registration", args=[self.event.short_url]), {"search": user.login_inp_net}
        )
        self.assertEqual(response.status_code, 200)
        self.assertTrue(Registration.objects.get(pk=reg.pk).used)
        self.assertContains(response, "Billet valide.")

    @tag("fast", "local")
    def test_checkByQRCode(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.restrictedticket, owner=user, paid=True)
        reg.save()
        response = self.client.post(
            reverse(
                "check-registration-with-uuid", kwargs={"event_id": self.event.id, "registration_uuid": reg.qrcode_uuid}
            ),
            {},
        )
        self.assertEqual(response.status_code, 200)
        self.assertTrue(Registration.objects.get(pk=reg.pk).used)
        self.assertContains(response, "Billet valide.")

    @tag("fast", "local")
    def test_checkByQRCodeWhenAlreadyChecked(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.restrictedticket, owner=user, paid=True)
        reg.save()
        self.client.post(
            reverse(
                "check-registration-with-uuid", kwargs={"event_id": self.event.id, "registration_uuid": reg.qrcode_uuid}
            ),
            {},
        )
        self.assertTrue(Registration.objects.get(pk=reg.pk).used)

        response = self.client.get(
            reverse(
                "check-registration-with-uuid", kwargs={"event_id": self.event.id, "registration_uuid": reg.qrcode_uuid}
            )
        )

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Ce billet est déjà validé".encode("utf-8"))
        self.assertNotContains(response, "Valider ce billet")

    @tag("fast", "local")
    def test_checkNotPaid(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.restrictedticket, owner=user, paid=False)
        reg.save()
        response = self.client.post(
            reverse("check-registration", args=[self.event.short_url]), {"search": user.login_inp_net}
        )
        self.assertEqual(response.status_code, 200)
        self.assertFalse(Registration.objects.get(pk=reg.pk).used)
        self.assertNotContains(response, "Billet valide.")
        self.assertContains(response, "Ce billet n’a pas été payé".encode("utf-8"))

    @tag("fast", "local")
    def test_checkWithoutAllInfoSup(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.openticket, owner=user, paid=True)
        reg.save()
        info = Info_Sup_Recv.objects.get(reference=self.infoSup1, registration=reg)
        info.state = Info_Sup_Recv.STATE_ACCEPTED
        info.save()
        reg.save()
        maj_state_registration(reg)
        response = self.client.post(
            reverse("check-registration", args=[self.event.short_url]), {"search": user.login_inp_net}
        )
        self.assertEqual(response.status_code, 200)
        self.assertFalse(Registration.objects.get(pk=reg.pk).used)
        self.assertNotContains(response, "Billet valide.")
        self.assertContains(response, "tous les documents demandés".encode("utf-8"))

    @tag("fast", "local")
    def test_checkOpposed(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.restrictedticket, owner=user, paid=True, opposed=True)
        reg.save()
        response = self.client.post(
            reverse("check-registration", args=[self.event.short_url]), {"search": user.login_inp_net}
        )
        self.assertEqual(response.status_code, 200)
        self.assertFalse(Registration.objects.get(pk=reg.pk).used)
        self.assertNotContains(response, "Billet valide.")
        self.assertContains(response, "Ce billet est en opposition".encode("utf-8"))

    @tag("fast", "local")
    def test_checkTestTemplate(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        response = self.client.post(
            reverse("check-registration", args=[self.event.short_url]), {"search": "%{}%%%%".format(self.openticket.id)}
        )
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, "Billet valide.")
        self.assertContains(response, "Billet de test (invalide, ne pas laisser entrer)".encode("utf-8"))

    @tag("fast", "local")
    def test_checkTestTemplateWithoutTicket(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        response = self.client.post(reverse("check-registration", args=[self.event.short_url]), {"search": "%0%%%%"})
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, "Billet valide.")

    @tag("fast", "local")
    def test_checkDoubleEntry(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        reg = Registration(ticket=self.restrictedticket, owner=user, paid=True)
        reg.save()
        self.client.post(reverse("check-registration", args=[self.event.short_url]), {"search": user.login_inp_net})
        response = self.client.post(
            reverse("check-registration", args=[self.event.short_url]), {"search": user.login_inp_net}
        )
        self.assertEqual(response.status_code, 200)
        self.assertTrue(Registration.objects.get(pk=reg.pk).used)
        self.assertNotContains(response, "Billet valide.")
        self.assertContains(response, "Le titulaire de ce billet est déjà entré à".encode("utf-8"))

    @tag("fast", "local")
    def test_checkLoginINPnetWithoutRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        response = self.client.post(
            reverse("check-registration", args=[self.event.short_url]), {"search": user.login_inp_net}
        )
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, "Billet valide.")
        self.assertContains(response, "Cet utilisateur n’a pas de billet pour cet évènement".encode("utf-8"))

    @tag("fast", "local")
    def test_checkQRCodeWithFalseRegistration(self):
        user = User.objects.get(login_inp_net="testone")
        self.client.force_login(user)
        response = self.client.post(
            reverse(
                "check-registration-with-uuid", kwargs={"event_id": self.event.id, "registration_uuid": uuid.uuid4()}
            ),
            {},
        )
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, "Billet valide.")
        self.assertContains(response, "Ce billet n’existe pas".encode("utf-8"))


class TestEventExport(CleanUp, CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    def setUp(self):
        self.tz = pytz.timezone("Europe/Paris")
        self.now = timezone.now().astimezone(self.tz)
        self.event = Event.objects.get(pk=1)
        self.event.tickets.all().delete()
        self.event.start_date = (self.now + datetime.timedelta(days=1)).astimezone(self.tz)
        self.event.end_date = (self.now + datetime.timedelta(days=2)).astimezone(self.tz)
        self.event.save()
        self.ticket = Ticket(
            event=self.event,
            name="OpenTicket",
            id=Ticket.next_id(self.event),
            short_notes="short description",
            long_notes="long description",
            price=10.0,
            online_payment=False,
            allowed_external=True,
            limit=0,
            admin_ticket=False,
            allow_anonyme=False,
        )
        self.ticket.save()
        # option
        self.optalone1 = Option(
            name="OptAlone1",
            price=5,
            description="Alone is alone",
            group_name=None,
            group=None,
            ticket=self.ticket,
            admin_only=False,
            mandatory=False,
        )
        self.optalone1.save()
        self.optalone2 = Option(
            name="OptAlone2",
            price=10,
            description="Alone is alone",
            group_name=None,
            group=None,
            ticket=self.ticket,
            admin_only=False,
            mandatory=False,
        )
        self.optalone2.save()
        # options group
        self.optgrp1c1 = Option(
            name="Choice1",
            price=15,
            description="Group is grouped",
            group_name="GroupOPT1",
            group=1,
            ticket=self.ticket,
            admin_only=False,
            mandatory=False,
        )
        self.optgrp1c1.save()
        self.optgrp1c2 = Option(
            name="Choice2",
            price=10,
            description="Group is grouped",
            group_name="GroupOPT1",
            group=1,
            ticket=self.ticket,
            admin_only=False,
            mandatory=False,
        )
        self.optgrp1c2.save()
        # info Sup
        self.infosimple = Info_Sup(
            ticket=self.ticket,
            name="InfoSimple",
            description="Description Simple",
            after_payement=True,
            type_info=Info_Sup.INFO_SIMPLE,
            info_validate=False,
            info_editable=False,
        )
        self.infosimple.save()
        self.infoshorttext = Info_Sup(
            ticket=self.ticket,
            name="InfoShortText",
            description="Description Short Text",
            after_payement=True,
            type_info=Info_Sup.INFO_SHORT_TEXT,
            info_validate=False,
            info_editable=False,
        )
        self.infoshorttext.save()
        self.infolongtext = Info_Sup(
            ticket=self.ticket,
            name="InfoLongText",
            description="Description Long Text",
            after_payement=True,
            type_info=Info_Sup.INFO_LONG_TEXT,
            info_validate=False,
            info_editable=False,
        )
        self.infolongtext.save()
        self.infochoices = Info_Sup_Choice(
            ticket=self.ticket,
            name="InfoChoice",
            description="Description Choice",
            after_payement=True,
            type_info=Info_Sup.INFO_CHOICE,
            info_validate=False,
            info_editable=False,
        )
        self.infochoices.save()
        self.choice1 = Info_Sup_Choice_Choices(
            info_sup=self.infochoices,
            name="Choice 1",
        )
        self.choice1.save()
        self.choice2 = Info_Sup_Choice_Choices(
            info_sup=self.infochoices,
            name="Choice 2",
        )
        self.choice2.save()
        self.infoint = Info_Sup_Int(
            ticket=self.ticket,
            name="InfoInt",
            description="Description Int",
            after_payement=True,
            type_info=Info_Sup.INFO_INT,
            info_validate=False,
            info_editable=False,
            min_value=None,
            max_value=None,
        )
        self.infoint.save()
        self.infofloat = Info_Sup_Float(
            ticket=self.ticket,
            name="InfoFloat",
            description="Description Float",
            after_payement=True,
            type_info=Info_Sup.INFO_FLOAT,
            info_validate=False,
            info_editable=False,
            min_value=None,
            max_value=None,
        )
        self.infofloat.save()
        self.infofile = Info_Sup_File(
            ticket=self.ticket,
            name="InfoFile",
            description="Description File",
            after_payement=True,
            type_info=Info_Sup.INFO_FILE,
            info_validate=False,
            info_editable=False,
            file_name="file_name",
        )
        self.infofile.save()
        # User
        self.user4 = User.objects.get(login_inp_net="testfour")
        self.user2 = User.objects.get(login_inp_net="testwo")
        self.useradmin = User.objects.get(login_inp_net="testone")
        self.usersuperadmin = User.objects.get(email="admin@billetterie.inpt.fr")
        # Registration
        self.reg = Registration(
            ticket=self.ticket,
            owner=self.user4,
            payee=None,
            paid=False,
        )
        self.reg.save()
        self.reg.options.add(self.optalone2, self.optgrp1c1)
        self.reg.save()
        # file for test
        self.path_img = os.path.join(settings.BASE_DIR, "guichet", "tests", "ressources", "ticket_template.png")
        # add random info
        info_recv = self.reg.info_sup_received.get(reference=self.infosimple)
        info_recv.state = Info_Sup_Recv.STATE_PENDING
        info_recv.save()
        info_recv = self.reg.info_sup_received.get(reference=self.infoshorttext).get_sup_type()
        info_recv.state = Info_Sup_Recv.STATE_REJECTED
        info_recv.value = "a text example"
        info_recv.save()
        info_recv = self.reg.info_sup_received.get(reference=self.infochoices).get_sup_type()
        info_recv.state = Info_Sup_Recv.STATE_ACCEPTED
        info_recv.value = self.choice1
        info_recv.save()
        info_recv = self.reg.info_sup_received.get(reference=self.infofloat).get_sup_type()
        info_recv.value = 10.25
        info_recv.save()
        info_recv = self.reg.info_sup_received.get(reference=self.infofile).get_sup_type()
        info_recv.value = UploadedFile(open(self.path_img, "rb"), "test.png")
        info_recv.save()

    @tag("low", "local")
    def test_exportXLSX(self):
        self.client.force_login(self.useradmin)
        response = self.client.get(reverse("download-xlsx-registrations", args=[self.event.short_url]))
        self.assertEqual(response.status_code, 200)
        self.client.force_login(self.user4)
        response2 = self.client.get(reverse("download-xlsx-registrations", args=[self.event.short_url]))
        self.assertEqual(response2.status_code, 403)

    @tag("low", "local")
    def test_exportXLSXNullFile(self):
        self.client.force_login(self.useradmin)
        info_recv = self.reg.info_sup_received.get(reference=self.infofile).get_sup_type()
        info_recv.value = None
        info_recv.save()
        response = self.client.get(reverse("download-xlsx-registrations", args=[self.event.short_url]))
        self.assertEqual(response.status_code, 200)

    @tag("low", "local")
    def test_exportZip(self):
        self.client.force_login(self.useradmin)
        response = self.client.get(reverse("download-zip-registrations", args=[self.event.short_url]))
        self.assertEqual(response.status_code, 200)
        self.client.force_login(self.user4)
        response2 = self.client.get(reverse("download-zip-registrations", args=[self.event.short_url]))
        self.assertEqual(response2.status_code, 403)

    @tag("low", "local")
    def test_exportTar(self):
        self.client.force_login(self.useradmin)
        response = self.client.get(reverse("download-targz-registrations", args=[self.event.short_url]))
        self.assertEqual(response.status_code, 200)
        self.client.force_login(self.user4)
        response2 = self.client.get(reverse("download-targz-registrations", args=[self.event.short_url]))
        self.assertEqual(response2.status_code, 403)
